#include <stdio.h>
#include "Bitmap.h"

void BmpGetDimensions(const BitmapPtr bitmapP, UInt16 *widthP, UInt16 *heightP, UInt16 *rowbytesP)
{
  if (widthP)
  {
    *widthP = (bitmapP[0]<<8)|bitmapP[1];
  }

  if (heightP)
  {
    *heightP = (bitmapP[2]<<8)|bitmapP[3];
  }

  if (rowbytesP)
  {
    *rowbytesP = (bitmapP[4]<<8)|bitmapP[5];
  }
}

UInt8 BmpGetBitDepth(const BitmapPtr bitmapP)
{
  if (BmpGetVersion(bitmapP) == 0)
  {
    return 1;
  }
  return bitmapP[8];  
}

UInt8 BmpGetVersion(const BitmapPtr bitmapP)
{
  return bitmapP[9];
}

UInt16 BmpGetDensity(const BitmapPtr bitmapP)
{
  UInt8 version = BmpGetVersion(bitmapP);
  if (version < 3)
  {
    return kDensityLow;
  }
  else if (version == 3)
  {
    return (bitmapP[14]<<8)|bitmapP[15];
  }

  /*
    FIXME: versions > 3?
  */
  return 0;
}

BitmapPtr BmpGetNextBitmap(const BitmapPtr bitmapP)
{
  UInt8 version = BmpGetVersion(bitmapP);
  if (version == 0)
  {
    return NULL;
  }
  else if (version <= 2)
  {
    UInt32 offset = (bitmapP[10]<<8)|bitmapP[11];
    if (offset == 0)
    {
      return NULL;
    }

    if (version == 1)
    {
      return &bitmapP[offset<<2];
    }
    else
    {
      return &bitmapP[16+(offset<<2)];
    }
  }
  else if (version == 3)
  {
    UInt32 offset = (bitmapP[20]<<24)|(bitmapP[21]<<16)|(bitmapP[22]<<8)|bitmapP[23];
    if (offset == 0)
    {
      return NULL;
    }
    return &bitmapP[offset];
  }

  /*
    FIXME: versions > 3?
  */
  return NULL;
}

BitmapCompressionType BmpGetCompressionType(const BitmapPtr bitmapP)
{
  if (bitmapP[6] & 0x80)
  {
    UInt8 version = BmpGetVersion(bitmapP);
    if (version <= 1)
    {
      return BitmapCompressionTypeScanLine;
    }
    else if (version <= 3)
    {
      return (BitmapCompressionType)bitmapP[13];
    }
    /*
      FIXME: versions > 3
    */
  }

  return BitmapCompressionTypeNone;
}
