#include <windows.h>
#include <winerror.h>
#include <winsock.h>
#include "DataMgr.h"
#include "MemoryMgr.h"

DmOpenRef DmOpenDatabase(LPCTSTR fileName)
{
    DmOpenRef self = (DmOpenRef )malloc(sizeof(DmDatabase));
    memset(self, 0, sizeof(DmDatabase));

    self->m_fileHandle = CreateFileForMapping(fileName,GENERIC_READ,0,NULL,OPEN_EXISTING,0,INVALID_HANDLE_VALUE);
    if (self->m_fileHandle != INVALID_HANDLE_VALUE)
    {
        DWORD fileSize = GetFileSize(self->m_fileHandle, NULL);

        self->m_memHandle = CreateFileMapping(self->m_fileHandle, NULL, PAGE_READONLY,0,0,NULL);
        if (self->m_memHandle != NULL)
        {
            self->m_startOfFile = MapViewOfFile(self->m_memHandle, FILE_MAP_READ, 0, 0, 0);
            if (self->m_startOfFile != NULL)
            {
                UInt8 *p = self->m_startOfFile;
                int i;
                memcpy(self->m_header.name,p,32);
                self->m_header.attributes = ntohs(*(UInt16 *)(&p[32]));
                self->m_header.version = ntohs(*(UInt16 *)(&p[34]));
                self->m_header.creationDate = ntohl(*(UInt32 *)(&p[36]));
                self->m_header.modificationDate = ntohl(*(UInt32 *)(&p[40]));
                self->m_header.lastBackupDate = ntohl(*(UInt32 *)(&p[44]));
                self->m_header.modificationNumber = ntohl(*(UInt32 *)(&p[48]));
                self->m_header.appInfoID = ntohl(*(UInt32 *)(&p[52]));
                self->m_header.sortInfoID = ntohl(*(UInt32 *)(&p[56]));
                self->m_header.type = ntohl(*(UInt32 *)(&p[60]));
                self->m_header.creator = ntohl(*(UInt32 *)(&p[64]));
                self->m_header.uniqueIDSeed = ntohl(*(UInt32 *)(&p[68]));

                p = &self->m_startOfFile[72];
                self->m_numRecordLists = 0;
                for (;;)
                {
                    self->m_numRecordLists++;
                    if (*(UInt32 *)p == 0)
                    {
                        break;
                    }
                    p += 6;
                }

                p = &self->m_startOfFile[72];
                self->m_recordLists = (RecordListType *)malloc(sizeof(RecordListType) * self->m_numRecordLists);

                /*
                    Mark the last structure with some useless data so we
                    know if we've read it in successfully.
                */
                self->m_recordLists[self->m_numRecordLists-1].firstEntry = 1;

                for (i = 0; i < self->m_numRecordLists; i++)
                {
                    self->m_recordLists[i].nextRecordListID = ntohl(*(UInt32 *)(&p[0]));
                    self->m_recordLists[i].numRecords = ntohs(*(UInt16 *)(&p[4]));
                    self->m_recordLists[i].firstEntry = 0;
                    self->m_numRecordEntries += self->m_recordLists[i].numRecords;
                    p += 6;
                }


                if (self->m_recordLists[self->m_numRecordLists-1].firstEntry == 0)
                {
                    self->m_constructed = true;
                    if (self->m_numRecordEntries != 0)
                    {
                        int j;
                        i = 0;
                        self->m_recordEntries = (RecordEntryType *)malloc(sizeof(RecordEntryType)*self->m_numRecordEntries);
                        memset(self->m_recordEntries,0,self->m_numRecordEntries*sizeof(RecordEntryType));


                        for (j = 0; j < self->m_numRecordLists; j++)
                        {
                            int k;
                            for (k = 0; k < (self->m_recordLists[j].numRecords); k++)
                            {
                                self->m_recordEntries[i].localChunkID = (*(p++)<<24);
                                self->m_recordEntries[i].localChunkID |=(*(p++)<<16);
                                self->m_recordEntries[i].localChunkID |=(*(p++)<<8);
                                self->m_recordEntries[i].localChunkID |=(*(p++)<<0);

                                self->m_recordEntries[i].attributes = *(p++);
                                self->m_recordEntries[i].uniqueID[0] = *(p++);
                                self->m_recordEntries[i].uniqueID[1] = *(p++);
                                self->m_recordEntries[i].uniqueID[2] = *(p++);
                                i++;
                            }
                        }
                    }    
                }
            }
        }
        else
        {
            DWORD d = GetLastError();
            d = d;
        }

        if (self->m_constructed)
        {
            int i;
            self->m_maxRecordLength = 0;

            if (self->m_numRecordEntries)
            {
              for (i = 0; i < self->m_numRecordEntries-1; i++)
              {
                  self->m_recordEntries[i].recordLength = self->m_recordEntries[i+1].localChunkID - self->m_recordEntries[i].localChunkID;
                  if (self->m_recordEntries[i].recordLength > self->m_maxRecordLength)
                  {
                      self->m_maxRecordLength = self->m_recordEntries[i].recordLength;
                  }
              }
              self->m_recordEntries[self->m_numRecordEntries-1].recordLength = fileSize - self->m_recordEntries[self->m_numRecordEntries-1].localChunkID;

              if (self->m_recordEntries[self->m_numRecordEntries-1].recordLength > self->m_maxRecordLength)
              {
                  self->m_maxRecordLength = self->m_recordEntries[self->m_numRecordEntries-1].recordLength;
              }
            }
            if (self->m_header.appInfoID)
            {
                if (self->m_header.sortInfoID)
                {
                    self->m_appInfoBlockLength = self->m_header.sortInfoID - self->m_header.appInfoID;
                }
                else /* if (m_numRecordEntries) - we must have some records, or m_constructed would have been false*/
                {
                    self->m_appInfoBlockLength = self->m_recordEntries[0].localChunkID - self->m_header.appInfoID;
                }
            }
        }
    }
    if (!self->m_constructed)
    {
      DmCloseDatabase(self);
      self = NULL;
    }
    return self;
}

MemHandle DmQueryRecord(DmOpenRef self, UInt16 index)
{
    if (index < self->m_numRecordEntries)
    {
#if 0
        MemHandle h = MemHandleNew(self->m_recordEntries[index].recordLength);
        memcpy(h->m_ptr, &self->m_startOfFile[self->m_recordEntries[index].localChunkID], h->m_size);
#else
        MemHandle h = MemHandleNewWithPointer(self->m_recordEntries[index].recordLength, &self->m_startOfFile[self->m_recordEntries[index].localChunkID]);
#endif

        return h;
    }
    return NULL;
}

Err DmCloseDatabase(DmOpenRef self)
{
    free(self->m_recordEntries);
    free(self->m_recordLists);
    UnmapViewOfFile(self->m_startOfFile);
    CloseHandle(self->m_memHandle);
    CloseHandle(self->m_fileHandle);
    free(self);
    return 0;
}


UInt16 DmNumRecords(DmOpenRef self)
{
    return self->m_numRecordEntries;
}

Err DmCreateDatabaseFromImage(LPCTSTR filename, const MemPtr bufferP, UInt32 size)
{
    HANDLE h = CreateFile(filename, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0, 0);
    if (h != INVALID_HANDLE_VALUE)
    {
        DWORD d;
        WriteFile (h, bufferP, size, &d, NULL);
        CloseHandle(h);
    }
    return 0;
}