#ifdef PDAV_SHOWGUIDE

#include "PDAVShowGuide.h"
#include "MapViewer.h"
#include "MemoryMgr.h"
#include "ExhibitorViewer.h"
#include "resrc1.h"
#include "library-ui.h"
#include "reader-ui.h"
#include "reader-toolbar.h"
#include <commctrl.h>

MapViewer *theMapViewer=NULL;

MapViewer *MapViewer_new(LPCTSTR dbName, Library *library)
{
    MapViewer *self = (MapViewer *)malloc(sizeof(MapViewer));
    memset(self, 0, sizeof(MapViewer));
    self->m_map = Map_new(dbName);
    self->m_highlightTimer = 0;
    self->m_library = library;

    return self;
}

void MapViewer_delete(MapViewer *self)
{
  if (self)
  {
      Map_delete(self->m_map);
      if (self->m_highlightTimer)
      {
        KillTimer(self->m_hwnd, self->m_highlightTimer);
      }
      DestroyWindow(self->m_toolbar);
      DestroyWindow(self->m_hwnd);
      free(self);
  }
}

void MapViewer_centre(MapViewer *self, const RECT *windowRect)
{
    int windowWidth = windowRect->right - windowRect->left;
    int windowHeight = windowRect->bottom - windowRect->top;

    self->m_xOffset = (windowWidth>>1) - (self->m_scale*self->m_xCentre);
    self->m_yOffset = (windowHeight>>1) - (self->m_scale*self->m_yCentre);
}

void MapViewer_initialise(MapViewer *self, const RECT *windowRect)
{
    int windowWidth, windowHeight, mapWidth, mapHeight;
    float xScale, yScale;
    
    windowWidth = windowRect->right - windowRect->left;
    windowHeight = windowRect->bottom - windowRect->top;
    mapWidth = self->m_map->m_mapRect.right - self->m_map->m_mapRect.left;
    mapHeight = self->m_map->m_mapRect.bottom - self->m_map->m_mapRect.top;

    xScale = (float)windowWidth / (float)mapWidth;
    yScale = (float)windowHeight / (float)mapHeight;

    if (xScale > yScale)
    {
        self->m_scale = yScale;
    }
    else
    {
        self->m_scale = xScale;
    }

    self->m_scale *= 0.9;

    self->m_xCentre = self->m_map->m_mapRect.left + (mapWidth>>1);
    self->m_yCentre = self->m_map->m_mapRect.top + (mapHeight>>1);

    MapViewer_centre(self, windowRect);
}

void MapViewer_show(MapViewer *self, Library *library, RECT *windowRect)
{
    TCHAR windowClass[64];
    RECT rect;

    LoadString(library->hInstance, IDS_MAP_VIEWER_CLASS, windowClass, 64 );

    self->m_hwnd = CreateWindow(windowClass, 
                                TEXT(""),
                                WS_BORDER | WS_POPUP,
                                windowRect->left,
                                windowRect->top,
                                windowRect->right - windowRect->left,
                                windowRect->bottom - windowRect->top,
                                library->hWnd,
                                NULL,
                                library->hInstance,
                                NULL);

    self->m_toolbar = CreateReaderToolbar(self->m_hwnd, library, true);

    GetWindowRect(self->m_hwnd,&rect);
    MapViewer_initialise(self,&rect);

    ShowWindow(self->m_hwnd, SW_SHOW);
    InvalidateRect(self->m_hwnd, NULL, TRUE);
    UpdateWindow(self->m_hwnd);
}

void MapViewer_setHighlight(MapViewer *self, RECT *highlightRect)
{
  if (self->m_highlightTimer)
  {
    KillTimer(self->m_hwnd, self->m_highlightTimer);
  }

  self->m_highlightRect = *highlightRect;
  self->m_highlightOn = true;
  self->m_highlightTimer = SetTimer(self->m_hwnd, 1, 500, NULL);
}

void MapViewer_mapRectToScreenRect(MapViewer *self, RECT *mapRect, RECT *screenRect)
{
    float fl,fr,ft,fb;

    fl = mapRect->left*self->m_scale;
    fr = mapRect->right*self->m_scale;
    ft = mapRect->top*self->m_scale;
    fb = mapRect->bottom*self->m_scale;

    screenRect->left = ((int)fl) + self->m_xOffset;
    screenRect->right = ((int)fr) + self->m_xOffset;
    screenRect->top = ((int)ft) + self->m_yOffset;
    screenRect->bottom = ((int)fb) + self->m_yOffset;
}

LRESULT CALLBACK MapViewer_WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    HDC hdc;

    switch (message) 
    {
        case WM_COMMAND:
        {
            WORD lo = LOWORD(wParam);
            if ((lo == ID_PREVIOUS_ITEM) || (lo == ID_NEXT_ITEM))
            {
              Library *library = theMapViewer->m_library;
              Boolean went = false;
              if (lo == ID_PREVIOUS_ITEM)
              {
                went = (HistoryList_goBack(library->history) != NULL);
              }
              else
              {
                went = (HistoryList_goForward(library->history) != NULL);
              }

              if (went)
              {
                MapViewer_delete(theMapViewer);
                GoCurrent(library, NULL);
              }
            }
            else if (lo == ID_GO_HOME)
            {
              Document *document;
              Library *library = theMapViewer->m_library;
              document = Library_findDocument( library, library->home_document );
              if( document != NULL )
              {
                MapViewer_delete(theMapViewer);
                OpenDocument( library->hWnd, document );
              }
            }
            else if (lo == ID_MAP_ZOOMIN)
            {
                RECT r;
                GetWindowRect(hWnd, &r);
                theMapViewer->m_scale *= 1.5;
                MapViewer_centre(theMapViewer, &r);
                InvalidateRect(hWnd, NULL, TRUE);
            }
            else if (lo == ID_MAP_ZOOMOUT)
            {
                RECT r;
                GetWindowRect(hWnd, &r);
                theMapViewer->m_scale /= 1.5;
                MapViewer_centre(theMapViewer, &r);
                InvalidateRect(hWnd, NULL, TRUE);
            }
            break;
        }
        case WM_NOTIFY:
        {
          LPNMHDR notify = (LPNMHDR)lParam;

          if( notify->hwndFrom == theMapViewer->m_toolbar )
          {
            switch( notify->code )
            {
              case NM_CUSTOMDRAW:
                return DrawToolbar( theMapViewer->m_toolbar, wParam, lParam );
            }
          }

          break;
        }
        case WM_LBUTTONDOWN:
        {
            theMapViewer->m_xMouseDown = LOWORD(lParam); 
            theMapViewer->m_yMouseDown = HIWORD(lParam); 
            theMapViewer->m_mouseMovedSinceButtonDown = false;
            break;
        }
        case WM_TIMER:
        {
          RECT r;
          MapViewer_mapRectToScreenRect(theMapViewer, &theMapViewer->m_highlightRect, &r);
          theMapViewer->m_highlightOn = !theMapViewer->m_highlightOn;
          InvalidateRect(theMapViewer->m_hwnd, &r, TRUE);
          break;
        }
        case WM_LBUTTONUP:
        {
            if (!theMapViewer->m_mouseMovedSinceButtonDown)
            {
                Int32 x, y;
                float fx, fy;
                UInt32 i;
                Map *theMap = theMapViewer->m_map;
                Boolean broken = false;

                x = (Int32) LOWORD(lParam);
                y = (Int32) HIWORD(lParam);

                x -= theMapViewer->m_xOffset;
                y -= theMapViewer->m_yOffset;

                fx = x/theMapViewer->m_scale;
                fy = y/theMapViewer->m_scale;

                x = (Int32)fx;
                y = (Int32)fy;

                for (i = 0; (!broken) && (i < theMap->m_exhibitorRecordNumbersCount); i++)
                {
                    int recNum = theMap->m_exhibitorRecordNumbers[i];
                    MemHandle h = DmQueryRecord(theMap->m_database, recNum);
                    UInt8 *rec=MemHandleLock(h);
                    int len = MemHandleSize(h);
                    const UInt8 *p = &rec[10];

                    while (p < (&rec[len]))
                    {
                        MapToExhibitorName mapToExhibitorName;

                        mapToExhibitorName.m_recordType = *(p++);
                        mapToExhibitorName.m_recordLength = *(p++);
                        mapToExhibitorName.m_left = *(p++)<<8;
                        mapToExhibitorName.m_left |= *(p++);
                        mapToExhibitorName.m_top = *(p++)<<8;
                        mapToExhibitorName.m_top |= *(p++);
                        mapToExhibitorName.m_right = *(p++)<<8;
                        mapToExhibitorName.m_right |= *(p++);
                        mapToExhibitorName.m_bottom = *(p++)<<8;
                        mapToExhibitorName.m_bottom |= *(p++);
                        mapToExhibitorName.m_exhibitorName = p; p += strlen(p)+1;
                        mapToExhibitorName.m_recordTypeAndExhibitorRecordNum = p; p += strlen(p)+1;


                        if ((x >= mapToExhibitorName.m_left) &&
                            (x <= mapToExhibitorName.m_right) &&
                            (y >= mapToExhibitorName.m_top) &&
                            (y <= mapToExhibitorName.m_bottom))
                        {
                            TCHAR buf[100];
                            RECT r;
                            HMENU menu = CreatePopupMenu();


                            {
                                int len = mapToExhibitorName.m_recordTypeAndExhibitorRecordNum - mapToExhibitorName.m_exhibitorName;
                                int i;
                                if (len > 99)
                                {
                                    len = 99;
                                }

                                for (i = 0; i < len; i++)
                                {
                                    buf[i] = mapToExhibitorName.m_exhibitorName[i];
                                }
                                buf[len] = '\0';

                            }


                            AppendMenu(menu,MF_STRING,1,buf);
                            GetWindowRect(hWnd, &r);
                            if (TrackPopupMenu(menu,
                                TPM_CENTERALIGN|TPM_VCENTERALIGN|TPM_RETURNCMD,
                                LOWORD(lParam)+r.left,
                                HIWORD(lParam)+r.top,
                                0,
                                hWnd,
                                NULL))
                            {
                                int type, id;
                                if (sscanf(mapToExhibitorName.m_recordTypeAndExhibitorRecordNum,"^%u^%u",&type, &id)==2)
                                {
                                  Document *document;
                                  ExhibitorViewer_new(id);

                                  document = Library_findDocument( theMapViewer->m_library, TEMPORARY_PLUCKER_DB_NAME );
                                  if( document != NULL )
                                  {
                                    OpenDocument( theMapViewer->m_library->hWnd, document );
                                    HistoryList_setHistoryItemExhibitor(HistoryList_add( theMapViewer->m_library->history), TEMPORARY_PLUCKER_DB_NAME, id);
                                    MapViewer_delete(theMapViewer);

                                    broken = true;
                                  }
                                }
                            }
                            break;
                        }
                        if (((p-rec))&1) p++;
                    }
                    MemHandleUnlock(h);
                    MemHandleFree(h);
                }
            }
            break;
        }
        case WM_MOUSEMOVE:
        {
            Int32 x, y;
            x = (Int32) LOWORD(lParam); 
            y = (Int32) HIWORD(lParam);

            if ((x != theMapViewer->m_xMouseDown) || (y != theMapViewer->m_yMouseDown))
            {
                RECT r;
                Int32 xx = (theMapViewer->m_xMouseDown-x);
                Int32 yy = (theMapViewer->m_yMouseDown-y);
                theMapViewer->m_xCentre += (Int32)(xx/theMapViewer->m_scale);
                theMapViewer->m_yCentre += (Int32)(yy/theMapViewer->m_scale);
                theMapViewer->m_xMouseDown = x;
                theMapViewer->m_yMouseDown = y;
                theMapViewer->m_mouseMovedSinceButtonDown = true;

                GetWindowRect(hWnd, &r);
                MapViewer_centre(theMapViewer, &r);
                InvalidateRect(hWnd, NULL, TRUE);
            }
            break;
        }
        case WM_PAINT:
        {
            PAINTSTRUCT ps;
            int reclen = 0;
            UInt32 i;
            int updateLeftMap, updateRightMap, updateTopMap, updateBottomMap;
            Map *theMap = theMapViewer->m_map;

            hdc = BeginPaint(hWnd, &ps);
            SelectObject(hdc,GetStockObject(WHITE_BRUSH));
            SelectObject(hdc,GetStockObject(BLACK_PEN));

            updateLeftMap = (int)(((float) (ps.rcPaint.left - theMapViewer->m_xOffset)) / theMapViewer->m_scale);
            updateRightMap = (int)(((float) (ps.rcPaint.right - theMapViewer->m_xOffset)) / theMapViewer->m_scale);
            updateTopMap = (int)(((float) (ps.rcPaint.top - theMapViewer->m_yOffset)) / theMapViewer->m_scale);
            updateBottomMap = (int)(((float) (ps.rcPaint.bottom - theMapViewer->m_yOffset)) / theMapViewer->m_scale);


            for (i = 0; i < theMap->m_boothRecordNumbersCount; i++)
            {
                int recNum = theMap->m_boothRecordNumbers[i];
                MemHandle h = DmQueryRecord(theMap->m_database, recNum);
                UInt8 *rec=MemHandleLock(h);
                int len = MemHandleSize(h);
                const UInt8 *p = &rec[10];

                while (p < (&rec[len]))
                {
                    MapToBoothNumber mapToBoothNumber;

                    mapToBoothNumber.m_recordType = *(p++);
                    mapToBoothNumber.m_recordLength = *(p++);
                    mapToBoothNumber.m_left = *(p++)<<8;
                    mapToBoothNumber.m_left |= *(p++);
                    mapToBoothNumber.m_top = *(p++)<<8;
                    mapToBoothNumber.m_top |= *(p++);
                    mapToBoothNumber.m_right = *(p++)<<8;
                    mapToBoothNumber.m_right |= *(p++);
                    mapToBoothNumber.m_bottom = *(p++)<<8;
                    mapToBoothNumber.m_bottom |= *(p++);
                    mapToBoothNumber.m_unknown1 = *(p++)<<8;
                    mapToBoothNumber.m_unknown1 |= *(p++);
                    mapToBoothNumber.m_boothNumber = p; p += strlen(p)+1;


                    /*
                        Compare the rectangle to see if we're within the update rect.
                    */
                    if ((mapToBoothNumber.m_left <= updateRightMap) &&
                        (mapToBoothNumber.m_right >= updateLeftMap) &&
                        (mapToBoothNumber.m_top <= updateBottomMap) &&
                        (mapToBoothNumber.m_bottom >= updateTopMap))
                    {
                        RECT r;
                        float fl,fr,ft,fb;

                        fl = mapToBoothNumber.m_left*theMapViewer->m_scale;
                        fr = mapToBoothNumber.m_right*theMapViewer->m_scale;
                        ft = mapToBoothNumber.m_top*theMapViewer->m_scale;
                        fb = mapToBoothNumber.m_bottom*theMapViewer->m_scale;

                        r.left = ((int)fl) + theMapViewer->m_xOffset;
                        r.right = ((int)fr) + theMapViewer->m_xOffset;
                        r.top = ((int)ft) + theMapViewer->m_yOffset;
                        r.bottom = ((int)fb) + theMapViewer->m_yOffset;

                        Rectangle(hdc, r.left, r.top, r.right, r.bottom);

                        {
                            TCHAR buf[100];
                            const UInt8 *sp = mapToBoothNumber.m_boothNumber;
                            LPTSTR dp = buf;
                            int len = p - sp;
                            SIZE size;

                            if (len > 99)
                            {
                                len = 99;
                            }

                            do
                            {
                                *(dp++) = *(sp++);
                            } while (sp < &mapToBoothNumber.m_boothNumber[len]);
                        
                            buf[len--] = '\0';

                            GetTextExtentPoint(hdc, buf, len, &size);
                            if ((size.cx <= (r.right-r.left)) &&
                                (size.cy <= (r.bottom-r.top)))
                            {
                                DrawText(hdc, buf, len, &r, DT_CENTER|DT_VCENTER|DT_SINGLELINE);
                            }
                        }
                    }

                    /*
                        Wait 'til we've drawn the text to skip any extra NUL padding, or it
                        will show up in the image.
                    */
                    if (((p-rec))&1) p++;
                }
                MemHandleUnlock(h);
                MemHandleFree(h);
            }

            if (theMapViewer->m_highlightOn)
            {
              RECT r;
              float fl,fr,ft,fb;

              fl = theMapViewer->m_highlightRect.left*theMapViewer->m_scale;
              fr = theMapViewer->m_highlightRect.right*theMapViewer->m_scale;
              ft = theMapViewer->m_highlightRect.top*theMapViewer->m_scale;
              fb = theMapViewer->m_highlightRect.bottom*theMapViewer->m_scale;

              r.left = ((int)fl) + theMapViewer->m_xOffset;
              r.right = ((int)fr) + theMapViewer->m_xOffset;
              r.top = ((int)ft) + theMapViewer->m_yOffset;
              r.bottom = ((int)fb) + theMapViewer->m_yOffset;

              SelectObject(hdc, GetStockObject(BLACK_BRUSH));
              Rectangle(hdc, r.left, r.top, r.right, r.bottom);

            }

            EndPaint(hWnd, &ps);
            break;
        }
        default:
        {
            return DefWindowProc(hWnd, message, wParam, lParam);
        }
    }
    return 0;
}


#endif

