#include <memory.h>
#include "PluckerWriter.h"
#include "PDAVShowGuide.h"

static const UInt8 static_fileHeader[PLUCKER_WRITER_FILE_HEADER_LENGTH] = 
{
    /*
        name - 32 bytes
    */
    '\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0',
    '\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0',

    0x00, 0x00, /* attributes */
    0x00, 0x01, /* version */
    0x00, 0x00, 0x00, 0x00, /* creation date */
    0x00, 0x00, 0x00, 0x00, /* modification date */
    0x00, 0x00, 0x00, 0x00, /* last backup date */
    0x00, 0x00, 0x00, 0x00, /* modification number */
    0x00, 0x00, 0x00, 0x00, /* app info id */
    0x00, 0x00, 0x00, 0x00, /* sort info id */
    'D', 'a', 't', 'a', /* type */
    'S', 'h', 'w', 'g', /* creator */
    0x00, 0x00, 0x00, 0x00, /* unique id seed */

    /*
        Record list
    */
    0x00, 0x00, 0x00, 0x00, /* next record list id */
};

#if 0
{
    /* everything below here is document-dependent */

    0x00, 0x02, /* number of records in document, including index record */

    /*
        Record entries (8 bytes * number above)
    */
    0x00,0x00,0x00,0x5e, /* localID of index record */
    0x00, /* attributes of index record */
    0x00,0x00,0x00, /* unique id of index record */

    0x00,0x00,0x00,0x68,
    0x00,
    0x00,0x00,0x01,

    /*
        Index record
    */
    0x00,0x01,
    0x00,0x00, 
    0x00,0x01,
    0x00,0x00,0x00,0x02,

    /*
        home.html record
    */
    0x00,0x02, /* plucker uid */
    0x00,0x01, /* #paras */
    0x00,0x0a, /* length of data */
    0x00, /* PHTML */
    0x00, /* reserved */
    0x00,0x06, /* size */
    0x00,0x00, /* 3-lower order bits are paragraph spacing (pixels/2)*/
    0x00,0x11,0x07,'f','o','o',

};
#endif

MemPtr PluckerWriter_fileHeader(MemPtr p, UInt16 version, const Char *name)
{
    UInt8 *pp = (UInt8 *)p;
    memcpy(p, static_fileHeader, PLUCKER_WRITER_FILE_HEADER_LENGTH);
    if (name == NULL)
    {
      name = TEMPORARY_PLUCKER_DB_NAME_CHARS;
    }
    strcpy(p, name);

    pp[34] = (version >> 8) & 0xff;
    pp[35] = version & 0xff;

    return &pp[PLUCKER_WRITER_FILE_HEADER_LENGTH];
}

MemPtr PluckerWriter_uInt8(MemPtr p, UInt8 value)
{
    UInt8 *pp = (UInt8 *)p;
    *(pp++) = value;
    return pp;
}

MemPtr PluckerWriter_uInt16(MemPtr p, UInt16 value)
{
    UInt8 *pp = (UInt8 *)p;
    *(pp++) = (UInt8)((value>>8)&0xff);
    *(pp++) = (UInt8)((value>>0)&0xff);
    return pp;
}

MemPtr PluckerWriter_uInt32(MemPtr p, UInt32 value)
{
    UInt8 *pp = (UInt8 *)p;
    *(pp++) = (UInt8)((value>>24)&0xff);
    *(pp++) = (UInt8)((value>>16)&0xff);
    *(pp++) = (UInt8)((value>>8)&0xff);
    *(pp++) = (UInt8)((value>>0)&0xff);
    return pp;
}

MemPtr PluckerWriter_recordEntry(MemPtr p, LocalID localID, UInt8 attributes, UInt32 uniqueID)
{
    UInt8 *pp = (UInt8 *)p;
    pp = PluckerWriter_uInt32(pp,localID);
    pp = PluckerWriter_uInt8(pp,attributes);
    pp = PluckerWriter_uInt8(pp,(UInt8)((uniqueID>>16)&0xff));
    pp = PluckerWriter_uInt16(pp,(UInt16)(uniqueID&0xffff));
    return pp;
}

MemPtr PluckerWriter_indexRecord(MemPtr p, UInt16 numReserved, const PluckerWriterReserved *reserved)
{
    UInt16 i;
    UInt8 *pp = (UInt8 *)p;
    *(pp++) = 0x00;
    *(pp++) = 0x01;
    *(pp++) = 0x00;
    *(pp++) = 0x01;
    pp = PluckerWriter_uInt16(pp,numReserved);
    for (i = 0; i < numReserved; i++)
    {
        pp = PluckerWriter_uInt16(pp,reserved[i].name);
        pp = PluckerWriter_uInt16(pp,reserved[i].id);
    }
    return pp;
}

MemPtr PluckerWriter_dataRecordHeader(MemPtr p, UInt16 uid, UInt16 paragraphs, UInt16 size, PluckerWriterDataRecordType type)
{
    UInt8 *pp = (UInt8 *)p;
    pp = PluckerWriter_uInt16(pp, uid);
    pp = PluckerWriter_uInt16(pp, paragraphs);
    pp = PluckerWriter_uInt16(pp, size);
    *(pp++) = (UInt8)type;
    *(pp++) = 0x00;
    return pp;
}

MemPtr PluckerWriter_writeZTS(MemPtr p, const char *string)
{
    return PluckerWriter_writeBlock(p, (MemPtr)string, strlen(string));
}

MemPtr PluckerWriter_writeBlock(MemPtr p, MemPtr src, UInt16 len)
{
    UInt8 *pp = (UInt8 *)p;
    memcpy(p, src, len);
    return &pp[len];
}

MemPtr PluckerWriter_pageLinkBegins(MemPtr p, UInt16 recordID)
{
    UInt8 *pp = (UInt8 *)p;
    *(pp++) = 0x00;
    *(pp++) = PLUCKER_CTL_PAGE_LINK_BEGINS;
    *(pp++) = recordID >> 8;
    *(pp++) = recordID & 0xff;
    return pp;
}


MemPtr PluckerWriter_paragraphLinkBegins(MemPtr p, UInt16 recordID, UInt16 paragraphOffset)
{
    UInt8 *pp = (UInt8 *)p;
    *(pp++) = 0x00;
    *(pp++) = PLUCKER_CTL_PARAGRAPH_LINK_BEGINS;
    *(pp++) = recordID >> 8;
    *(pp++) = recordID & 0xff;
    *(pp++) = paragraphOffset >> 8;
    *(pp++) = paragraphOffset & 0xff;
    return pp;
}


MemPtr PluckerWriter_linkEnds(MemPtr p)
{
    UInt8 *pp = (UInt8 *)p;
    *(pp++) = 0x00;
    *(pp++) = PLUCKER_CTL_LINK_ENDS;
    return pp;
}


MemPtr PluckerWriter_setFont(MemPtr p, PluckerWriterFontSpecifier fontSpecifier)
{
    UInt8 *pp = (UInt8 *)p;
    *(pp++) = 0x00;
    *(pp++) = PLUCKER_CTL_SET_FONT;
    *(pp++) = fontSpecifier;
    return pp;
}


MemPtr PluckerWriter_embeddedImage(MemPtr p, UInt16 recordID)
{
    UInt8 *pp = (UInt8 *)p;
    *(pp++) = 0x00;
    *(pp++) = PLUCKER_CTL_EMBEDDED_IMAGE;
    *(pp++) = recordID >> 8;
    *(pp++) = recordID & 0xff;
    return pp;
}


MemPtr PluckerWriter_setMargin(MemPtr p, UInt8 leftMargin, UInt8 rightMargin)
{
    UInt8 *pp = (UInt8 *)p;
    *(pp++) = 0x00;
    *(pp++) = PLUCKER_CTL_SET_MARGIN;
    *(pp++) = leftMargin;
    *(pp++) = rightMargin;
    return pp;
}


MemPtr PluckerWriter_alignmentOfText(MemPtr p, PluckerWriterAlignment alignment)
{
    UInt8 *pp = (UInt8 *)p;
    *(pp++) = 0x00;
    *(pp++) = PLUCKER_CTL_ALIGNMENT_OF_TEXT;
    *(pp++) = alignment;
    return pp;
}


MemPtr PluckerWriter_horizontalRule(MemPtr p, UInt8 height, UInt8 widthPixels, UInt8 widthPercent)
{
    UInt8 *pp = (UInt8 *)p;
    *(pp++) = 0x00;
    *(pp++) = PLUCKER_CTL_HORIZONTAL_RULE;
    *(pp++) = height;
    *(pp++) = widthPixels;
    *(pp++) = widthPercent;
    return pp;
}


MemPtr PluckerWriter_newLine(MemPtr p)
{
    UInt8 *pp = (UInt8 *)p;
    *(pp++) = 0x00;
    *(pp++) = PLUCKER_CTL_NEW_LINE;
    return pp;
}


MemPtr PluckerWriter_italicTextBegins(MemPtr p)
{
    UInt8 *pp = (UInt8 *)p;
    *(pp++) = 0x00;
    *(pp++) = PLUCKER_CTL_ITALIC_TEXT_BEGINS;
    return pp;
}


MemPtr PluckerWriter_italicTextEnds(MemPtr p)
{
    UInt8 *pp = (UInt8 *)p;
    *(pp++) = 0x00;
    *(pp++) = PLUCKER_CTL_ITALIC_TEXT_ENDS;
    return pp;
}


MemPtr PluckerWriter_setTextColor(MemPtr p, UInt8 red, UInt8 green, UInt8 blue)
{
    UInt8 *pp = (UInt8 *)p;
    *(pp++) = 0x00;
    *(pp++) = PLUCKER_CTL_SET_TEXT_COLOR;
    *(pp++) = red;
    *(pp++) = green;
    *(pp++) = blue;
    return pp;
}


MemPtr PluckerWriter_multipleEmbeddedImage(MemPtr p, UInt16 alternateImageRecordID, UInt16 imageRecordID)
{
    UInt8 *pp = (UInt8 *)p;
    *(pp++) = 0x00;
    *(pp++) = PLUCKER_CTL_MULTIPLE_EMBEDDED_IMAGE;
    *(pp++) = alternateImageRecordID >> 8;
    *(pp++) = alternateImageRecordID & 0xff;
    *(pp++) = imageRecordID >> 8;
    *(pp++) = imageRecordID & 0xff;
    return pp;
}


MemPtr PluckerWriter_underlineTextBegins(MemPtr p)
{
    UInt8 *pp = (UInt8 *)p;
    *(pp++) = 0x00;
    *(pp++) = PLUCKER_CTL_UNDERLINE_TEXT_BEGINS;
    return pp;
}


MemPtr PluckerWriter_underlineTextEnds(MemPtr p)
{
    UInt8 *pp = (UInt8 *)p;
    *(pp++) = 0x00;
    *(pp++) = PLUCKER_CTL_UNDERLINE_TEXT_ENDS;
    return pp;
}


MemPtr PluckerWriter_strikeThroughTextBegins(MemPtr p)
{
    UInt8 *pp = (UInt8 *)p;
    *(pp++) = 0x00;
    *(pp++) = PLUCKER_CTL_STRIKE_THROUGH_TEXT_BEGINS;
    return pp;
}


MemPtr PluckerWriter_strikeThroughTextEnds(MemPtr p)
{
    UInt8 *pp = (UInt8 *)p;
    *(pp++) = 0x00;
    *(pp++) = PLUCKER_CTL_STRIKE_THROUGH_TEXT_ENDS;
    return pp;
}


MemPtr PluckerWriter_16bitUnicodeCharacter(MemPtr p, UInt8 alternateTextLength, UInt16 unicodeCharacter)
{
    UInt8 *pp = (UInt8 *)p;
    *(pp++) = 0x00;
    *(pp++) = PLUCKER_CTL_16_BIT_UNICODE_CHARACTER;
    *(pp++) = alternateTextLength;
    *(pp++) = unicodeCharacter >> 8;
    *(pp++) = unicodeCharacter & 0xff;
    return pp;
}


MemPtr PluckerWriter_32bitUnicodeCharacter(MemPtr p, UInt8 alternateTextLength, UInt32 unicodeCharacter)
{
    UInt8 *pp = (UInt8 *)p;
    *(pp++) = 0x00;
    *(pp++) = PLUCKER_CTL_32_BIT_UNICODE_CHARACTER;
    *(pp++) = alternateTextLength;
    *(pp++) = (UInt8)((unicodeCharacter >> 24)&0xff);
    *(pp++) = (UInt8)((unicodeCharacter >> 16)&0xff);
    *(pp++) = (UInt8)((unicodeCharacter >> 8)&0xff);
    *(pp++) = (UInt8)(unicodeCharacter & 0xff);
    return pp;
}
