#include "DataMgr.h"
#include "MemoryMgr.h"
#include "SessionSearch.h"
#include "Session.h"
#include "PluckerWriter.h"
#include "PDAVShowGuide.h"


SessionSearch *SessionSearch_new(int year, int month, int day, const Char *keyword, const Char *track, const Char *cme)
{
  SessionSearch *self;
  DmOpenRef db;
  UInt16 numRecords;
  UInt16 id;
  Boolean canBreak = false;
  int trackLen = strlen(track);
  int keywordLen = strlen(keyword);
  int cmeLen = strlen(cme);

  self = (SessionSearch *)malloc(sizeof(SessionSearch));
  self->m_itemsCount = 0;
  self->m_itemsSize = 16;
  self->m_items = (SessionSearchItem *)malloc(sizeof(SessionSearchItem)*self->m_itemsSize);
  memset(self->m_items, 0, sizeof(SessionSearchItem)*self->m_itemsSize);
  self->m_year = year;
  self->m_month = month;
  self->m_day = day;
  self->m_keyword = malloc(keywordLen+1); memcpy((Char *)self->m_keyword, keyword, keywordLen+1);
  self->m_track = malloc(trackLen+1); memcpy((Char *)self->m_track, track, trackLen+1);
  self->m_cme = malloc(cmeLen+1); memcpy((Char *)self->m_cme, cme, cmeLen+1);
  


  db = DmOpenDatabase(SESSIONS_DB_NAME);
  numRecords = DmNumRecords(db);

  for (id = 0; id < numRecords; id++)
  {
    MemHandle hrec;
    const UInt8 *prec;
    const UInt8 *p;
    Session session;
    Boolean match = true;

    hrec = DmQueryRecord(db, id);
    prec = MemHandleLock(hrec);

    p = prec;

    session.m_eventNo = p; p += strlen(p) + 1;
    session.m_date = p; p += strlen(p) + 1;
    session.m_year = p; p += strlen(p) + 1;
    session.m_month = p; p += strlen(p) + 1;
    session.m_day = p; p += strlen(p) + 1;
    session.m_startText = p; p += strlen(p) + 1;
    session.m_endText = p; p += strlen(p) + 1;
    session.m_startTime = p; p += strlen(p) + 1;
    session.m_endTime = p; p += strlen(p) + 1;
    session.m_cme = p; p += strlen(p) + 1;
    session.m_keyword = p; p += strlen(p) + 1;
    session.m_event = p; p += strlen(p) + 1;
    session.m_sponsorship = p; p += strlen(p) + 1;
    session.m_supportedBy = p; p += strlen(p) + 1;
    session.m_fullName = p; p += strlen(p) + 1;
    session.m_facility = p; p += strlen(p) + 1;
    session.m_location = p; p += strlen(p) + 1;
    session.m_seqNum = p; p += strlen(p) + 1;
    session.m_track = p;

    if (session.m_cme[0] == '\0')
    {
      match = false;
    }

    if ((match) && (month != 0))
    {
      int sessday, sessmonth, sessyear;
      sessday = atoi(session.m_day);
      sessmonth = atoi(session.m_month);
      sessyear = atoi(session.m_year);
      match = false;

      if (sessyear > year)
      {
        canBreak = true;
      }
      else if (sessyear == year)
      {
        if (sessmonth > month)
        {
          canBreak = true;
        }
        else if (sessmonth == month)
        {
          if (sessday > day)
          {
            canBreak = true;
          }
          else if (sessday == day)
          {
            match = true;
          }
        }
      }
    }

    if (match)
    {
      if (cme[0] != '\0')
      {
        if (strcmp(cme, session.m_cme))
        {
          match = false;
        }
      }
    }

    if (match)
    {
      if (trackLen)
      {
        match = false;

        while (*p != '\0')
        {
          int result = strncmp(track, p, trackLen);
          if (result == 0)
          {
            match = true;
            break;
          }
          else if (result < 0)
          {
            break;
          }
          else
          {
            while ((*p != ' ') && (*p != '\0'))
            {
              p++;
            }

            if (*p == ' ')
            {
              p++;
            }
          }
        }
      }
    }

    if (match)
    {
      if (keywordLen)
      {
        match = false;
        p = session.m_keyword;

        while (*p != '\0')
        {
          int result = strncmp(keyword, p, keywordLen);
          if (result == 0)
          {
            match = true;
            break;
          }
          else if (result < 0)
          {
            break;
          }
          else
          {
            while ((*p != ' ') && (*p != '\0'))
            {
              p++;
            }

            if (*p == ' ')
            {
              p++;
            }
          }
        }
      }
    }

    if (match)
    {

      if (self->m_itemsCount == self->m_itemsSize)
      {
        UInt32 newSize;
        SessionSearchItem *newItems;

        if (self->m_itemsSize > 64)
        {
          newSize = self->m_itemsSize + 16;
        }
        else
        {
          newSize = self->m_itemsSize << 1;
        }

        newItems = (SessionSearchItem *)malloc(sizeof(SessionSearchItem) * newSize);
        memcpy(newItems, self->m_items, sizeof(SessionSearchItem) * self->m_itemsSize);
        free(self->m_items);
        self->m_items = newItems;
        self->m_itemsSize = newSize;
      }

      self->m_items[self->m_itemsCount].description = (Char *)malloc(session.m_sponsorship - session.m_event);
      strcpy(self->m_items[self->m_itemsCount].description,session.m_event);
      self->m_items[self->m_itemsCount].id = id;
      self->m_itemsCount++;
    }

    MemHandleUnlock(hrec);
    MemHandleFree(hrec);

    if (canBreak)
    {
      break;
    }
  }

  DmCloseDatabase(db);

  return self;
}

void SessionSearch_delete(SessionSearch *self)
{
  UInt32 i;
  for (i = 0; i < self->m_itemsCount; i++)
  {
    free(self->m_items[i].description);
  }
  free(self->m_items);
  free(self->m_keyword);
  free(self->m_track);
  free(self->m_cme);
  free(self);
}

void SessionSearch_generatePlucker(SessionSearch *self)
{
    UInt32 len;
    UInt16 paragraphLen = 0;
    LocalID indexRecordLocalID;
    LocalID homeHTMLRecordLocalID;
    LocalID urlHandlingRecordLocalID;
    LocalID urlDataRecordLocalID;
    const char *urlTransport = "session:";
    const char *headingString = "Search Results:";

    UInt16 urlLen;
    UInt16 urlId = 0;
    UInt32 index;

    int i;
    PluckerWriterReserved reserved[]=
    {
        {PLUCKER_RESERVED_HOME_HTML, 2},
        {PLUCKER_RESERVED_URL_HANDLING, 3},
    };
#define ARRAY_ELEMENTS(x) sizeof(x)/sizeof(x[0])

    paragraphLen = (2*PLUCKER_CTL_LEN_SET_TEXT_COLOR)+(PLUCKER_CTL_LEN_SET_FONT+PLUCKER_CTL_LEN_SET_FONT+PLUCKER_CTL_LEN_NEW_LINE);
    paragraphLen += strlen(headingString);

    paragraphLen += PLUCKER_CTL_LEN_NEW_LINE;
    paragraphLen += PLUCKER_CTL_LEN_SET_MARGIN;

    for (index = 0; index < self->m_itemsCount; index++)
    {
        int tmpLen = strlen(self->m_items[index].description);
        if (tmpLen > 30)
        {
            paragraphLen += 33;
        }
        else
        {
            paragraphLen += tmpLen;
        }

        paragraphLen += PLUCKER_CTL_LEN_PAGE_LINK_BEGINS + PLUCKER_CTL_LEN_LINK_ENDS;
        paragraphLen += PLUCKER_CTL_LEN_NEW_LINE;
        urlId++;

    }

    /*
        Basic header and Record List
    */
    len = PLUCKER_WRITER_FILE_HEADER_LENGTH+2;
    len += PLUCKER_WRITER_RECORD_ENTRY_LENGTH * (2+(ARRAY_ELEMENTS(reserved)));

    indexRecordLocalID = len;
    len += PLUCKER_WRITER_INDEX_RECORD_LENGTH(ARRAY_ELEMENTS(reserved));

    homeHTMLRecordLocalID = len;
    len += PLUCKER_WRITER_DATA_RECORD_HEADER_LENGTH;
    len += PLUCKER_WRITER_TEXT_DATA_RECORD_PARAGRAPH_HEADER_LENGTH*1;

    len += paragraphLen;

    /*
        Data size all taken in to account. Now work out the size for the URLs
    */
    urlHandlingRecordLocalID = len;
    len += PLUCKER_WRITER_DATA_RECORD_HEADER_LENGTH;
    len += PLUCKER_WRITER_URL_HANDLING_DATA_RECORD_PARAGRAPH_HEADER_LENGTH;

    urlDataRecordLocalID = len;
    len += PLUCKER_WRITER_DATA_RECORD_HEADER_LENGTH;
    len += 4; // 4 zero terminators for unused records

    urlLen = (strlen(urlTransport)+5+1)*urlId;

    len += urlLen;

    urlId = 5;

    {

        UInt8 *dbPtr = malloc(len);
        UInt8 *tmp = dbPtr;

        tmp = PluckerWriter_fileHeader(tmp, 1, NULL);
        tmp = PluckerWriter_uInt16(tmp, 4); /* num record entries */
        tmp = PluckerWriter_recordEntry(tmp, indexRecordLocalID, 0, 0);
        tmp = PluckerWriter_recordEntry(tmp, homeHTMLRecordLocalID, 0, 1);
        tmp = PluckerWriter_recordEntry(tmp, urlHandlingRecordLocalID, 0, 2);
        tmp = PluckerWriter_recordEntry(tmp, urlDataRecordLocalID, 0, 3);
        tmp = PluckerWriter_indexRecord(tmp, ARRAY_ELEMENTS(reserved), reserved);
        tmp = PluckerWriter_dataRecordHeader(tmp, reserved[0].id, 1, paragraphLen+(PLUCKER_WRITER_TEXT_DATA_RECORD_PARAGRAPH_HEADER_LENGTH*1), PLUCKER_DATATYPE_PHTML);
        tmp = PluckerWriter_uInt16(tmp, paragraphLen);
        tmp = PluckerWriter_uInt16(tmp, 0);

        tmp = PluckerWriter_setFont(tmp, PLUCKER_FONT_BOLD);
        tmp = PluckerWriter_setTextColor(tmp, 0xff, 0x80, 0x00);
        tmp = PluckerWriter_writeZTS(tmp, headingString);
        tmp = PluckerWriter_setTextColor(tmp, 0x00, 0x00, 0x00);
        tmp = PluckerWriter_setFont(tmp, PLUCKER_FONT_REGULAR);
        tmp = PluckerWriter_newLine(tmp);

        tmp = PluckerWriter_newLine(tmp);
        tmp = PluckerWriter_setMargin(tmp, 4, 0);

        for (index = 0; index < self->m_itemsCount; index++)
        {
            int tmpLen;
            tmp = PluckerWriter_pageLinkBegins(tmp, urlId++);

            tmpLen = strlen(self->m_items[index].description);
            if (tmpLen > 30)
            {
                Char tmpStr[34];
                memcpy(tmpStr,self->m_items[index].description,30);
                strcpy(&tmpStr[30],"...");
                tmp = PluckerWriter_writeZTS(tmp, tmpStr);
            }
            else
            {
                tmp = PluckerWriter_writeZTS(tmp, self->m_items[index].description);
            }

            tmp = PluckerWriter_linkEnds(tmp);
            tmp = PluckerWriter_newLine(tmp);
        }

        /*
            URL handling record.
        */
        tmp = PluckerWriter_dataRecordHeader(tmp, reserved[1].id, 0, PLUCKER_WRITER_URL_HANDLING_DATA_RECORD_PARAGRAPH_HEADER_LENGTH, PLUCKER_DATATYPE_LINK_INDEX);
        tmp = PluckerWriter_uInt16(tmp, urlId);
        tmp = PluckerWriter_uInt16(tmp, 4);

        /*
            URL data record.
        */
        tmp = PluckerWriter_dataRecordHeader(tmp, 4, 0, urlLen+4, PLUCKER_DATATYPE_LINKS);
        tmp = PluckerWriter_uInt8(tmp, 0); // NUL terminator (URL 1)
        tmp = PluckerWriter_uInt8(tmp, 0); // NUL terminator (URL 2)
        tmp = PluckerWriter_uInt8(tmp, 0); // NUL terminator (URL 3)
        tmp = PluckerWriter_uInt8(tmp, 0); // NUL terminator (URL 4)

        urlId -= 5;
        for (i = 0; i < urlId; i++)
        {
            Char buf[8];
            tmp = PluckerWriter_writeZTS(tmp, urlTransport);
            sprintf(buf, "%05u",self->m_items[i].id);
            tmp = PluckerWriter_writeZTS(tmp, buf);
            tmp = PluckerWriter_uInt8(tmp, 0); // NUL terminator
        }

        {
            int dbLen = tmp - dbPtr;
            dbLen = dbLen;
        }

        DmCreateDatabaseFromImage(TEMPORARY_PLUCKER_DB_NAME, dbPtr, len);
        free(dbPtr);
    }

}