/*>=-----------------------------------------------------------------------=<*
 * Vade Mecum: bookmarks module
 * Copyright 2003 by Jamis Buck
 * ------------------------------------------------------------------------
 * This file is part of Vade Mecum.
 *
 * Vade Mecum is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Vade Mecum is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Vade Mecum; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *>=-----------------------------------------------------------------------=<*/

#include "bookmarks.h"
#include "simple-db.h"


#define BOOKMARK_DATA_LEN(h) (int)( (char*)&(h)->dbid - (char*)(h) )


typedef struct
{
  SimpleDB db;
}
BookmarkManagerRec;


static int static_bookmark_comparer( void*, void* );


BookmarkManager BookmarkManager_create( char *filename )
{
  BookmarkManagerRec *rec;
  SimpleDBDefinition def;
  Bookmark bkmark;

  def.filename = filename;
  def.record_size = BOOKMARK_DATA_LEN( &bkmark );
  def.variable_sized_records = FALSE;

  rec = (BookmarkManagerRec*)malloc( sizeof( BookmarkManagerRec ) );
  rec->db = SimpleDB_open( &def );

  return (BookmarkManager)rec;
}


void BookmarkManager_destroy( BookmarkManager bm )
{
  BookmarkManagerRec *rec = (BookmarkManagerRec*)bm;

  SimpleDB_close( rec->db );
  rec->db = NULL;
  free( rec );
}


LinkedList* BookmarkManager_getBookmarks( BookmarkManager bm )
{
  BookmarkManagerRec *rec = (BookmarkManagerRec*)bm;
  LinkedList *list;
  Bookmark  buffer;
  SimpleDBEnumerator iter;
  SimpleDB db;
  int id;

  db = rec->db;
  list = LinkedList_new( sizeof( Bookmark ), static_bookmark_comparer );

  while( ( id = SimpleDB_enumerate( db, &buffer, BOOKMARK_DATA_LEN(&buffer), &iter ) ) >= 0 )
  {
    buffer.dbid = id;
    LinkedList_insert( list, &buffer );

    db = NULL;
  }

  return list;
}


void BookmarkManager_addBookmark( BookmarkManager bm,
                                  LinkedList *bookmarks,
                                  Bookmark *bookmark )
{
  BookmarkManagerRec *rec = (BookmarkManagerRec*)bm;

  bookmark->dbid = SimpleDB_insert( rec->db, bookmark, BOOKMARK_DATA_LEN(bookmark) );
  LinkedList_insert( bookmarks, bookmark );
}


void BookmarkManager_deleteBookmark( BookmarkManager bm,
                                     LinkedList *bookmarks,
                                     Bookmark *bookmark )
{
  BookmarkManagerRec *rec = (BookmarkManagerRec*)bm;
  LinkedListNode *node;

  SimpleDB_delete( rec->db, bookmark->dbid );

  node = LinkedList_findNode( bookmarks, bookmark );
  LinkedList_deleteNode( bookmarks, node );
}


void BookmarkManager_modifyBookmark( BookmarkManager bm,
                                     Bookmark *bookmark,
                                     int record_id,
                                     int char_pos,
                                     int delta_y,
                                     TCHAR* name )
{
  BookmarkManagerRec *rec = (BookmarkManagerRec*)bm;

  bookmark->record_id = record_id;
  bookmark->char_pos = char_pos;
  bookmark->delta_y = delta_y;
  wcscpy( bookmark->name, name );

  SimpleDB_update( rec->db, bookmark->dbid, bookmark, BOOKMARK_DATA_LEN(bookmark) );
}



static int static_bookmark_comparer( void* v1, void* v2 )
{
  Bookmark *h1 = (Bookmark*)v1;
  Bookmark *h2 = (Bookmark*)v2;

  return _wcsicmp( h1->name, h2->name );
}
