/*>=-----------------------------------------------------------------------=<*
 * Vade Mecum: colorbox control
 * Copyright 2003 by Jamis Buck
 * ------------------------------------------------------------------------
 * This file is part of Vade Mecum.
 *
 * Vade Mecum is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Vade Mecum is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Vade Mecum; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 * ------------------------------------------------------------------------
 * The colormap control defines a small box of a specific color.  The color
 * may be modified.  When the control has focus, a focus rectangle will
 * surround it.  When the control loses focus, the focus rectangle will go
 * away.  When the rectangle is clicked, it will emit a CLBXN_CLICKED
 * notification message (via the WM_COMMAND event).
 *
 * Author: Jamis Buck (jgb3@email.byu.edu)
 *>=-----------------------------------------------------------------------=<*/

#ifndef __COLORBOX_CTL_H__
#define __COLORBOX_CTL_H__


#include <windows.h>


/* get the current color of the color box.
 *   wParam = lParam = 0
 *   returns COLORREF
 */
#define CLBXM_GET_COLOR       ( WM_USER + 1 )

/* set the current color of the color box.
 *   wParam = COLORREF
 *   lParam = 0
 */
#define CLBXM_SET_COLOR       ( WM_USER + 2 )

/* set whether or not the color box is selected.
 *   wParam = BOOL
 *   lParam = 0
 */
#define CLBXM_SELECTED        ( WM_USER + 3 )

/* query whether or not the color box is selected.
 *   wParam = lParam = 0
 *   returns BOOL
 */
#define CLBXM_IS_SELECTED     ( WM_USER + 4 )


/* notification message emitted when the control is clicked. (sent via the
 * WM_COMMAND event.) */
#define CLBXN_CLICKED         (  1 )


/* registers the color box control class */
ATOM RegisterColorBoxControl( HINSTANCE hInst );

/* creates a new color box with the default color (black). */
HWND CreateColorBox( HWND hParent,
                     HINSTANCE hInst,
                     DWORD style,
                     int x, int y,
                     int width, int height,
                     DWORD id );

#endif /* __COLORBOX_CTL_H__ */
