/*>=-----------------------------------------------------------------------=<*
 * Vade Mecum: folder browser dialog
 * Copyright 2003 by Jamis Buck
 * ------------------------------------------------------------------------
 * This file is part of Vade Mecum.
 *
 * Vade Mecum is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Vade Mecum is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Vade Mecum; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *>=-----------------------------------------------------------------------=<*/

#include <windows.h>
#include <aygshell.h>
#include <commctrl.h>
#include <dbgapi.h>

#include "folder_browser.h"

#define FOLDER_BROWSER_CLASS    L"jgb3FolderBrowser"

#define FBD_MARGIN             ( 5 )

#define ID_DIRECTORY_TREE      ( 40001 )


static BOOL CALLBACK FolderBrowserProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );
static HWND CreateFolderBrowserDlg();
static void PopulateDirectoryTree( HWND hTree );
static BOOL PopulateSubTree( HWND hTree, HTREEITEM subtree, TCHAR *path, BOOL query_subtrees );
static void ComputePathName( HWND hTree, HTREEITEM subtree, TCHAR *path );

static BOOL FolderBrowserDlg_OnCreate( HWND hWnd, WPARAM wParam, LPARAM lParam );
static BOOL FolderBrowserDlg_OnNotify( HWND hWnd, WPARAM wParam, LPARAM lParam );
static BOOL FolderBrowserDlg_OnCommand( HWND hWnd, WPARAM wParam, LPARAM lParam );


static FOLDERBROWSERINFO *g_folder_info = NULL;


ATOM RegisterFolderBrowser( HINSTANCE hInst )
{
  WNDCLASS wc;

  wc.style = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc = FolderBrowserProc;
  wc.cbClsExtra = 0;
  wc.cbWndExtra = 0;
  wc.hInstance = hInst;
  wc.hIcon = (HICON)NULL;
  wc.hbrBackground = (HBRUSH)GetStockObject( WHITE_BRUSH );
  wc.hCursor = (HCURSOR)NULL;
  wc.lpszMenuName = NULL;
  wc.lpszClassName = FOLDER_BROWSER_CLASS;

  return RegisterClass( &wc );
}


BOOL GetFolderName( FOLDERBROWSERINFO *fbi )
{
  HWND hBrowser;
  MSG msg;

  g_folder_info = fbi;

  hBrowser = CreateFolderBrowserDlg();
  SHFullScreen( hBrowser, SHFS_HIDESIPBUTTON );
  EnableWindow( fbi->hParent, FALSE );

  ShowWindow( hBrowser, SW_SHOW );
  UpdateWindow( hBrowser );

  while( GetMessage( &msg, NULL, 0, 0 ) )
  {
    TranslateMessage( &msg );
    DispatchMessage( &msg );
  }

  EnableWindow( fbi->hParent, TRUE );

  return msg.wParam;
}


static BOOL CALLBACK FolderBrowserProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
  switch( uMsg )
  {
    case WM_CREATE:
      return FolderBrowserDlg_OnCreate( hWnd, wParam, lParam );

    case WM_NOTIFY:
      return FolderBrowserDlg_OnNotify( hWnd, wParam, lParam );

    case WM_COMMAND:
      return FolderBrowserDlg_OnCommand( hWnd, wParam, lParam );
  }

  return DefWindowProc( hWnd, uMsg, wParam, lParam );
}


static HWND CreateFolderBrowserDlg()
{
  HWND hFolder;
  int cx;
  int cy;
  int width;
  int height;
  HDC dc;

  dc = GetDC( g_folder_info->hParent );
  cx = GetDeviceCaps( dc, HORZRES );
  cy = GetDeviceCaps( dc, VERTRES );
  ReleaseDC( g_folder_info->hParent, dc );

  width = (int)( cx * 0.9 );
  height = (int)( cy * 0.75 );

  hFolder = CreateWindow( FOLDER_BROWSER_CLASS,
                          ( g_folder_info->title != NULL ? g_folder_info->title : L"Select Folder" ),
                          WS_VISIBLE | WS_POPUP | WS_CAPTION,
                          ( cx - width ) / 2, ( cy - height ) / 2,
                          width, height,
                          g_folder_info->hParent, (HMENU)NULL, g_folder_info->hInstance, NULL );

  return hFolder;
}


static BOOL FolderBrowserDlg_OnCreate( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
  HWND ctl;
  int  xofs;
  int  yofs;
  int  width;
  int  height;
  RECT rect;
  int  cx;
  int  cy;
  SHINITDLGINFO shidi;

  shidi.dwMask = SHIDIM_FLAGS;
  shidi.hDlg = hWnd;
  shidi.dwFlags = SHIDIF_DONEBUTTON | SHIDIF_SIZEDLG;
  SHInitDialog( &shidi );

  GetClientRect( hWnd, &rect );
  cx = ( rect.right - rect.left );
  cy = ( rect.bottom - rect.top );

  xofs = FBD_MARGIN;
  yofs = FBD_MARGIN;

  /* first, create some text describing the treeview */
  width = 100;
  height = 20;
  ctl = CreateWindow( L"static", L"Directory Tree:", WS_VISIBLE | SS_LEFTNOWORDWRAP,
                      xofs, yofs+3, width, height-3, hWnd, (HMENU)NULL,
                      g_folder_info->hInstance, NULL );

  /* create a cancel button */
  xofs = cx - FBD_MARGIN - 50;
  width = 50;
  ctl = CreateWindow( L"button", L"Cancel", WS_VISIBLE | BS_PUSHBUTTON,
                      xofs, yofs, width, height, hWnd, (HMENU)IDCANCEL,
                      g_folder_info->hInstance, NULL );

  /* create the directory tree */
  xofs = FBD_MARGIN;
  yofs += height + 4;
  width = cx - FBD_MARGIN*2;
  height = cy - FBD_MARGIN - yofs;
  ctl = CreateWindow( WC_TREEVIEW, NULL, WS_VISIBLE | WS_BORDER |
                        TVS_HASLINES | TVS_LINESATROOT | TVS_HASBUTTONS,
                      xofs, yofs, width, height, hWnd, (HMENU)ID_DIRECTORY_TREE,
                      g_folder_info->hInstance, NULL );

  PopulateDirectoryTree( ctl );

  /* expand the root */
  TreeView_Expand( ctl, TreeView_GetChild( ctl, NULL ), TVE_EXPAND );

  return TRUE;
}


static BOOL FolderBrowserDlg_OnNotify( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
  NMHDR *hdr = (NMHDR*)lParam;
  NM_TREEVIEW *ntv;
  TCHAR path[ MAX_PATH ];

  switch( hdr->code )
  {
    case TVN_ITEMEXPANDING:
      ntv = (NM_TREEVIEW*)lParam;

      /* if the node already has children, don't repopulate it */
      if( TreeView_GetChild( hdr->hwndFrom, ntv->itemNew.hItem ) == NULL )
      {
        ComputePathName( hdr->hwndFrom, ntv->itemNew.hItem, path );
        PopulateSubTree( hdr->hwndFrom, ntv->itemNew.hItem, path, FALSE );
      }
      return FALSE;
  }

  return DefWindowProc( hWnd, WM_NOTIFY, wParam, lParam );
}


static BOOL FolderBrowserDlg_OnCommand( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
  HWND hTree;
  HTREEITEM sel;

  switch( LOWORD( wParam ) )
  {
    case IDOK:
      hTree = GetDlgItem( hWnd, ID_DIRECTORY_TREE );
      sel = TreeView_GetSelection( hTree );
      if( sel == NULL )
      {
        MessageBox( hWnd, L"You must select a folder before tapping OK.", L"Error",
                    MB_OK | MB_ICONEXCLAMATION);
        return TRUE;
      }

      ComputePathName( hTree, sel, g_folder_info->folder_path );

      PostQuitMessage( TRUE );
      DestroyWindow( hWnd );
      return TRUE;

    case IDCANCEL:
      PostQuitMessage( FALSE );
      DestroyWindow( hWnd );
      return TRUE;
  }

  return DefWindowProc( hWnd, WM_COMMAND, wParam, lParam );
}


static void PopulateDirectoryTree( HWND hTree )
{
  TVINSERTSTRUCT tvis;
  HTREEITEM root;

  /* define the root item */
  tvis.hParent = TVI_ROOT;
  tvis.hInsertAfter = TVI_FIRST;
  tvis.item.mask = TVIF_CHILDREN | TVIF_TEXT;
  tvis.item.cChildren = 1;
  tvis.item.pszText = L"My Device";

  root = TreeView_InsertItem( hTree, &tvis );
  PopulateSubTree( hTree, root, L"", FALSE );
}


static BOOL PopulateSubTree( HWND hTree, HTREEITEM subtree, TCHAR *path, BOOL query_subtrees )
{
  WIN32_FIND_DATA data;
  HANDLE hSearch;
  TCHAR pattern[MAX_PATH];
  TCHAR tempfname[MAX_PATH];
  BOOL result = FALSE;
  BOOL has_subtrees;
  TVINSERTSTRUCT tvis;

  wcscpy( pattern, path );
  wcscat( pattern, L"\\*.*" ); /* match all files in the given path */

  /* if query_subtrees is TRUE, then all we do return TRUE if this subtree has
   * any subfolders, and return FALSE otherwise.  If query_subtrees is FALSE,
   * we actually populate 'subtree' with all of its immediate subtrees. */

  hSearch = FindFirstFile( pattern, &data );
  if( hSearch == INVALID_HANDLE_VALUE )
  {
    return result; /* immediately -- no matches found */
  }

  do
  {
    if( ( data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY ) == FILE_ATTRIBUTE_DIRECTORY )
    {
      result = TRUE;

      if( query_subtrees )
      {
        break;
      }

      wcscpy( tempfname, path );
      wcscat( tempfname, L"\\" );
      wcscat( tempfname, data.cFileName );

      has_subtrees = PopulateSubTree( hTree, NULL, tempfname, TRUE );

      tvis.hParent = subtree;
      tvis.hInsertAfter = TVI_SORT;
      tvis.item.mask = TVIF_CHILDREN | TVIF_TEXT;
      tvis.item.cChildren = ( has_subtrees ? 1 : 0 );
      tvis.item.pszText = data.cFileName;
      TreeView_InsertItem( hTree, &tvis );
    }
  }
  while( FindNextFile( hSearch, &data ) );

  FindClose( hSearch );
  return result;
}


static void ComputePathName( HWND hTree, HTREEITEM subtree, TCHAR *path )
{
  HTREEITEM parent;
  TVITEM item;
  TCHAR name[128];

  parent = TreeView_GetParent( hTree, subtree );
  if( parent == NULL )
  {
    path[0] = 0;
  }
  else
  {
    ComputePathName( hTree, parent, path );

    item.mask = TVIF_TEXT | TVIF_HANDLE ;
    item.pszText = name;
    item.cchTextMax = 128;
    item.hItem = subtree;

    TreeView_GetItem( hTree, &item );

    wcscat( path, L"\\" );
    wcscat( path, item.pszText );
  }
}

