/*>=-----------------------------------------------------------------------=<*
 * Vade Mecum: folder browser module
 * Copyright 2003 by Jamis Buck
 * ------------------------------------------------------------------------
 * This file is part of Vade Mecum.
 *
 * Vade Mecum is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Vade Mecum is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Vade Mecum; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 * ------------------------------------------------------------------------
 * This defines a more flexible interface by which a user may navigate a
 * device's directory hierarchy, and ultimately select a single folder
 * (the name of which will be returned in the appropriate field(s) of the
 * associated structure).
 *
 * Author: Jamis Buck (jgb3@email.byu.edu)
 *>=-----------------------------------------------------------------------=<*/

#ifndef __FOLDER_BROWSER_H__
#define __FOLDER_BROWSER_H__

#include <windows.h>

/* This structure defines the information necessary to construct a folder browser
 * window. */
typedef struct
{
  TCHAR    *folder_path;      /* pointer to a TCHAR buffer which will contain the name of the selected folder */
  UINT      folder_path_len;  /* the size of the folder_path buffer, in characters */
  TCHAR    *title;            /* the title of the folder browser window */
  HWND      hParent;          /* the window that owns this folder browser instance */
  HINSTANCE hInstance;        /* the handle to the module that owns this folder browser instance */
} FOLDERBROWSERINFO;


/**
 * Registers the window class for the folder browser window.  This should be called when the
 * application starts, before a folder browser is instantiated.
 */
ATOM RegisterFolderBrowser( HINSTANCE hInst );

/**
 * Open a folder browser instance defined by the given information.  Returns TRUE if the user selected a
 * folder and clicked 'OK' (in which case the 'folder_path' field of the structure will contain the full
 * path of the selected folder).  Returns FALSE if the user canceled the operation, in which case the
 * contents of the structure are undefined.
 */
BOOL GetFolderName( FOLDERBROWSERINFO *fbi );

#endif /* __FOLDER_BROWSER_H__ */
