/*>=-----------------------------------------------------------------------=<*
 * Vade Mecum: getUserText module
 * Copyright 2003 by Jamis Buck
 * ------------------------------------------------------------------------
 * This file is part of Vade Mecum.
 *
 * Vade Mecum is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Vade Mecum is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Vade Mecum; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 * ------------------------------------------------------------------------
 * This creates a window that allows users to enter text, either a single
 * line or multiple lines.
 *
 * Author: Jamis Buck (jgb3@email.byu.edu)
 *>=-----------------------------------------------------------------------=<*/

#ifndef __GET_TEXT_H__
#define __GET_TEXT_H__

#include <windows.h>


/* This defines all the information necessary to create a GetUserText window */
typedef struct
{
  TCHAR     *title;      /* the title of the window */
  TCHAR     *prompt;     /* the prompt that is displayed above the EDIT control */
  UINT       width;      /* the width (in pixels) of the edit control */
  UINT       height;     /* the height (in pixels) of the edit control */
  UINT       flags;      /* style flags that are passed to the EDIT control when it is created */
  HWND       hWnd;       /* the window that will own the window */
  HINSTANCE  hInstance;  /* the module that will own the window */

  TCHAR     *text;       /* the buffer to recieve the entered/edited text.  This also defines the initial value of the control */
  UINT       text_len;   /* maximum number of characters that may be copied to 'text' (including null byte). */
} GETUSERTEXTOPTS;


/**
 * Registers the GetUserText window class.  This should be called before GetUserText is ever called.
 */
ATOM RegisterGetTextWindow( HINSTANCE hInst );

/**
 * Creates a GetUserText window defined by the given options.  The window will be sized to fit the requested size of
 * the EDIT control, if possible; otherwise, the edit control will be sized to fit the maximum possible window size.
 * Returns TRUE if the user pressed the 'OK' button, or FALSE if the user closed the window by pressing the 'X' button.
 * If the user pressed 'OK', the 'text' field of the structure will contain up to 'text_len' bytes of the entered
 * text (possibly truncated).
 */
BOOL GetUserText( GETUSERTEXTOPTS *opts );

#endif /* __GET_TEXT_H__ */
