/*>=-----------------------------------------------------------------------=<*
 * Vade Mecum: glyph cache module
 * Copyright 2003 by Jamis Buck
 * ------------------------------------------------------------------------
 * This file is part of Vade Mecum.
 *
 * Vade Mecum is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Vade Mecum is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Vade Mecum; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 * ------------------------------------------------------------------------
 * The glyph cache stores metric information about a single glyph in a given
 * font.  Like the font cache, it stores the glyphs in MRU order.  The
 * glyph cache is typically managed by the font cache, since when a font
 * gets expired, any glyphs associated with it in the glyph cache should be
 * thrown away as well.
 *
 * Author: Jamis Buck (jgb3@email.byu.edu)
 *>=-----------------------------------------------------------------------=<*/

#ifndef __GLYPH_CACHE_H__
#define __GLYPH_CACHE_H__

#include <wingdi.h> /* LOGINFO, etc. */


/* an opaque GlyphCache data type */
typedef void *GlyphCache;

typedef struct _glyph_info_ GlyphInfo;

/* information about a single glyph */
struct _glyph_info_
{
  unsigned char width;   /* glyph width (in pixels) */
  unsigned char height;  /* glyph height (in pixels) */
};


#ifdef __cplusplus
extern "C" {
#endif

/**
 * Instantiate a new glyph cache capable of holding information about 'size' different glyphs.
 * Return NULL if the GlyphCache cannot be created.
 */
GlyphCache GlyphCache_new( int size );

/**
 * Delete the given cache and release any memory associated with it.
 */
void GlyphCache_delete( GlyphCache cache );

/**
 * Invalidates all glyph cache entries associated with the given font.  If the font is NULL, all
 * glyph cache entries are invalidated (removed).
 */
void GlyphCache_invalidate( GlyphCache cache,
                            HFONT font );

/**
 * Retrieves glyph information about a single character in a font.  The GlyphInfo reference that
 * is returned should not be modified or freed.
 */
GlyphInfo* GlyphCache_get( GlyphCache cache,
                           HDC hdc,
                           HFONT font,
                           TCHAR c );

#ifdef __cplusplus
}
#endif

#endif /* __GLYPH_CACHE_H__ */
