/*>=-----------------------------------------------------------------------=<*
 * Vade Mecum: highlights module
 * Copyright 2003 by Jamis Buck
 * ------------------------------------------------------------------------
 * This file is part of Vade Mecum.
 *
 * Vade Mecum is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Vade Mecum is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Vade Mecum; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *>=-----------------------------------------------------------------------=<*/

#include "highlights.h"
#include "simple-db.h"


#define HIGHLIGHT_DATA_LEN(h) (int)( (char*)&(h)->dbid - (char*)(h) )


typedef struct
{
  SimpleDB db;
}
HighlightManagerRec;


static int static_highlight_comparer( void*, void* );



HighlightManager HighlightManager_create( char *filename )
{
  HighlightManagerRec *rec;
  SimpleDBDefinition def;
  Highlight h;

  def.filename = filename;
  def.record_size = HIGHLIGHT_DATA_LEN( &h );
  def.variable_sized_records = FALSE;

  rec = (HighlightManagerRec*)malloc( sizeof( HighlightManagerRec ) );
  rec->db = SimpleDB_open( &def );

  return (HighlightManager)rec;
}


void HighlightManager_destroy( HighlightManager hm )
{
  HighlightManagerRec *rec = (HighlightManagerRec*)hm;

  SimpleDB_close( rec->db );
  rec->db = NULL;
  free( rec );
}


LinkedList* HighlightManager_getHighlights( HighlightManager hm,
                                            int record_id )
{
  HighlightManagerRec *rec = (HighlightManagerRec*)hm;
  LinkedList *list;
  Highlight  buffer;
  SimpleDBEnumerator iter;
  SimpleDB db;
  int id;

  db = rec->db;
  list = LinkedList_new( sizeof( Highlight ), static_highlight_comparer );

  while( ( id = SimpleDB_enumerate( db, &buffer, HIGHLIGHT_DATA_LEN(&buffer), &iter ) ) >= 0 )
  {
    if( buffer.record_id == record_id )
    {
      buffer.dbid = id;
      LinkedList_insert( list, &buffer );
    }

    db = NULL;
  }

  return list;
}


void HighlightManager_addHighlight( HighlightManager hm,
                                    LinkedList *highlights,
                                    Highlight *highlight )
{
  HighlightManagerRec *rec = (HighlightManagerRec*)hm;

  highlight->dbid = SimpleDB_insert( rec->db, highlight, HIGHLIGHT_DATA_LEN(highlight) );

  LinkedList_insert( highlights, highlight );
}


void HighlightManager_deleteHighlight( HighlightManager hm,
                                       LinkedList *highlights,
                                       Highlight *highlight )
{
  HighlightManagerRec *rec = (HighlightManagerRec*)hm;
  LinkedListNode *node;

  SimpleDB_delete( rec->db, highlight->dbid );

  node = LinkedList_findNode( highlights, highlight );
  LinkedList_deleteNode( highlights, node );
}


void HighlightManager_modifyHighlight( HighlightManager hm,
                                       Highlight *highlight,
                                       int fg,
                                       int bg,
                                       int annotation )
{
  HighlightManagerRec *rec = (HighlightManagerRec*)hm;

  highlight->fg = fg;
  highlight->bg = bg;
  highlight->annotation = annotation;

  SimpleDB_update( rec->db, highlight->dbid, highlight, HIGHLIGHT_DATA_LEN(highlight) );
}



static int static_highlight_comparer( void* v1, void* v2 )
{
  Highlight *h1 = (Highlight*)v1;
  Highlight *h2 = (Highlight*)v2;

  if( h1->start_pos < h2->start_pos ) return -1;
  if( h1->start_pos > h2->start_pos ) return  1;

  if( h1->end_pos < h2->end_pos ) return -1;
  if( h1->end_pos > h2->end_pos ) return  1;

  return 0;
}
