#include <stdlib.h>

#include "history.h"


HISTORY_LIST *HistoryList_new( int capacity )
{
  HISTORY_LIST *list;

  list = (HISTORY_LIST*)malloc( sizeof( HISTORY_LIST ) );
  memset( list, 0, sizeof( *list ) );

  list->capacity = capacity;

  return list;
}


void HistoryList_delete( HISTORY_LIST *list )
{
  HISTORY_ITEM *item;

  item = list->head;
  while( item != NULL )
  {
    HISTORY_ITEM *next = item->next;
    free( item );
    item = next;
  }

  memset( list, 0, sizeof( *list ) );
  free( list );
}

#ifdef PDAV_SHOWGUIDE

void HistoryList_setHistoryItemPlucker(HISTORY_ITEM *hi,
                                       LPCTSTR filename,
                                       int record,
                                       int char_index,
                                       int y_offset )
{
  hi->type = HISTORY_ITEM_TYPE_PLUCKER;
  _tcsncpy(hi->current_document, filename, MAX_PATH);
  hi->item.plucker.char_index = char_index;
  hi->item.plucker.y_offset = y_offset;
  hi->item.plucker.record = record;
}

void HistoryList_setHistoryItemMap(HISTORY_ITEM *hi,
                                   float zoomLevel,
                                   Int32 xCentre,
                                   Int32 yCentre,
                                   RECT *highlightRect)
{
  hi->type = HISTORY_ITEM_TYPE_MAP;
  hi->current_document[0] = '\0';
  hi->item.map.zoomLevel = zoomLevel;
  hi->item.map.xCentre = xCentre;
  hi->item.map.yCentre = yCentre;
  hi->item.map.highlightRect = *highlightRect;
}

void HistoryList_setHistoryItemSession(HISTORY_ITEM *hi,
                                       LPCTSTR filename,
                                       UInt16 id)
{
  hi->type = HISTORY_ITEM_TYPE_SESSION;
  _tcsncpy(hi->current_document, filename, MAX_PATH);
  hi->item.session.id = id;
}

void HistoryList_setHistoryItemSessionList(HISTORY_ITEM *hi,
                                           LPCTSTR filename,
                                           int year, int month, int day, const char *keyword, const char *track, const char *cme)
{
  hi->type = HISTORY_ITEM_TYPE_SESSION_LIST;
  _tcsncpy(hi->current_document, filename, MAX_PATH);
  hi->item.sessionList.year = year;
  hi->item.sessionList.month = month;
  hi->item.sessionList.day = day;
  strcpy(hi->item.sessionList.keyword, keyword);
  strcpy(hi->item.sessionList.track, track);
  strcpy(hi->item.sessionList.cme, cme);
}

void HistoryList_setHistoryItemSessionSearch(HISTORY_ITEM *hi,int year, int month, int day, const char *keyword, const char *track, const char *cme)
{
  hi->type = HISTORY_ITEM_TYPE_SESSION_SEARCH;
  hi->item.sessionSearch.year = year;
  hi->item.sessionSearch.month = month;
  hi->item.sessionSearch.day = day;
  strcpy(hi->item.sessionSearch.keyword, keyword);
  strcpy(hi->item.sessionSearch.track, track);
  strcpy(hi->item.sessionSearch.cme, cme);
}



void HistoryList_setHistoryItemExhibitor(HISTORY_ITEM *hi,
                                         LPCTSTR filename,
                                         UInt16 id)
{
  hi->type = HISTORY_ITEM_TYPE_EXHIBITOR;
  _tcsncpy(hi->current_document, filename, MAX_PATH);
  hi->item.exhibitor.id = id;
}

void HistoryList_setHistoryItemExhibitorList(HISTORY_ITEM *hi,
                                             LPCTSTR filename,
                                             UInt16 startId,
                                             char limitChar)
{
  hi->type = HISTORY_ITEM_TYPE_EXHIBITOR_LIST;
  _tcsncpy(hi->current_document, filename, MAX_PATH);
  hi->item.exhibitorList.startId = startId;
  hi->item.exhibitorList.limitChar = limitChar;
}


HISTORY_ITEM* HistoryList_add( HISTORY_LIST *list )

#else

HISTORY_ITEM* HistoryList_add( HISTORY_LIST *list,
                               int record,
                               int char_index,
                               int y_offset )
#endif
{
  HISTORY_ITEM *c;
  HISTORY_ITEM *i;

  c = (HISTORY_ITEM*)malloc( sizeof( HISTORY_ITEM ) );
  memset( c, 0, sizeof( *c ) );

#ifndef PDAV_SHOWGUIDE
  c->char_index = char_index;
  c->y_offset = y_offset;
  c->record = record;
#endif

  c->previous = list->current;
  c->next = NULL;

  list->length++;

  /* if the list has outgrown the capacity, delete the head of the list (or the tail, if the current is
   * the head) and make the head (or tail) the next (or previous) item in the list. */

  if( list->length > list->capacity )
  {
    list->length--;

    if( list->current != list->head )
    {
      i = list->head;
      list->head = list->head->next;
      list->head->previous = NULL;
    }
    else
    {
      i = list->tail;
      list->tail = list->tail->previous;
      list->tail->next = NULL;
    }

    free( i );
  }

  /* if list->tail is NULL, then the list is empty, and the new item becomes the list. */

  if( list->tail == NULL )
  {
    list->head = list->tail = list->current = c;
  }
  else
  {
    /* otherwise, delete all items that follow 'current' and make the new item
     * list->current, list->current->next, and list->tail. */

    i = list->current->next;
    while( i != NULL )
    {
      HISTORY_ITEM *n = i->next;
      free( i );
      i = n;

      list->length--;
    }

    list->current->next = c;
    list->tail = c;
    list->current = c;
  }

  return c;
}


HISTORY_ITEM *HistoryList_current( HISTORY_LIST *list )
{
  return list->current;
}


HISTORY_ITEM *HistoryList_goBack( HISTORY_LIST *list )
{
  if( list->current->previous != NULL )
  {
    list->current = list->current->previous;
    return list->current;
  }

  return NULL;
}


HISTORY_ITEM *HistoryList_goForward( HISTORY_LIST *list )
{
  if( list->current->next != NULL )
  {
    list->current = list->current->next;
    return list->current;
  }

  return NULL;
}


int HistoryList_priorElementsExist( HISTORY_LIST *list )
{
  return ( list->current != NULL && list->current->previous != NULL );
}


int HistoryList_subsequentElementsExist( HISTORY_LIST *list )
{
  return ( list->current != NULL && list->current->next != NULL );
}


int HistoryList_serialize( HISTORY_LIST *list,
                           FILE *file )
{
  HISTORY_ITEM *i;

  fwrite( &list->length, sizeof( list->length ), 1, file );

  for( i = list->head; i != NULL; i = i->next )
  {
    char is_current = ( list->current == i );

    fwrite( &is_current, sizeof( is_current ), 1, file );
    fwrite( i, sizeof( int ), 3, file );
  }

  return 0;
}


HISTORY_LIST* HistoryList_unserialize( FILE *file,
                                       int capacity )
{
  HISTORY_LIST *list;
  int index;

  list = HistoryList_new( capacity );

  fread( &list->length, sizeof( list->length ), 1, file );
  for( index = 0; index < list->length; index++ )
  {
    HISTORY_ITEM *i;
    char is_current;

    i = (HISTORY_ITEM*)malloc( sizeof( HISTORY_ITEM ) );

    fread( &is_current, sizeof( is_current ), 1, file );
    fread( i, sizeof( int ), 3, file );

    if( list->tail ) list->tail->next = i;

    i->previous = list->tail;
    list->tail = i;
    i->next = NULL;

    if( list->head == NULL ) list->head = i;    
    if( is_current ) list->current = i;
  }

  if( list->length > list->capacity )
  {
    HISTORY_ITEM *i;

    /* first, delete from the head towards the tail.  If we encounter the current element,
     * stop and begin deleting from the tail towards the head. */

    i = list->head;
    while( list->length > list->capacity && i != NULL && i != list->current )
    {
      HISTORY_ITEM *n = i->next;

      n->previous = NULL;
      free( i );
      i = n;

      list->length--;
    }

    i = list->tail;
    while( list->length > list->capacity && i != NULL && i != list->current )
    {
      HISTORY_ITEM *n = i->previous;

      n->next = NULL;
      free( i );
      i = n;

      list->length--;
    }
  }

  return list;
}