/*>=-----------------------------------------------------------------------=<*
 * Vade Mecum: linked list module
 * Copyright 2003 by Jamis Buck
 * ------------------------------------------------------------------------
 * This file is part of Vade Mecum.
 *
 * Vade Mecum is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Vade Mecum is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Vade Mecum; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *>=-----------------------------------------------------------------------=<*/

#include <stdio.h>

#include "linked-list.h"


LinkedList* LinkedList_new( int data_size, LinkedListCompareFn compare )
{
  LinkedList *list;

  list = (LinkedList*)malloc( sizeof( LinkedList ) );
  list->list = NULL;
  list->data_size = data_size;
  list->compare = compare;

  return list;
}


void LinkedList_delete( LinkedList* list )
{
  LinkedListNode *node;
  LinkedListNode *next;

  node = list->list;
  while( node != NULL )
  {
    next = node->next;
    free( node );
    node = next;
  }

  memset( list, 0, sizeof( *list ) );
  free( list );
}


void LinkedList_insert( LinkedList* list, void* data )
{
  LinkedListNode *l, *tail, *item;
  int found = 0;
  int rc;

  item = (LinkedListNode*)malloc( sizeof( LinkedListNode ) + list->data_size );
  item->next = item->prev = NULL;
  memcpy( NODEDATA( item, void* ), data, list->data_size );

  tail = NULL;
  l = list->list;
  while( l != NULL )
  {
    if( list->compare != NULL )
    {
      rc = list->compare( NODEDATA( item, void* ), NODEDATA( l, void* ) );
      if( rc < 0 )
      {
        found = 1;
        break;
      }
    }

    tail = l;
    l = l->next;
  }

  if( !found ) /* insert at end of list */
  {
    if( !tail ) /* list is empty! */
    {
      list->list = item;
    }
    else
    {
      item->prev = tail;
      tail->next = item;
    }
  }
  else
  {
    if( l->prev == NULL ) /* insert at head of list */
    {
      l->prev = item;
      item->next = l;
      list->list = item;
    }
    else
    {
      item->prev = l->prev;
      l->prev->next = item;
      l->prev = item;
      item->next = l;
    }
  }
}


void LinkedList_deleteNode( LinkedList* list, LinkedListNode *node )
{
  if( list->list == node )
  {
    list->list = node->next;
  }

  if( node->prev )
  {
    node->prev->next = node->next;
  }

  if( node->next )
  {
    node->next->prev = node->prev;
  }

  memset( node, 0, sizeof( LinkedListNode ) + list->data_size );
  free( node );
}


LinkedListNode* LinkedList_findNode( LinkedList *list, void *data )
{
  LinkedListNode *node;

  node = list->list;
  while( node != NULL )
  {
    if( memcmp( NODEDATA(node,void*), data, list->data_size ) == 0 )
    {
      return node;
    }

    node = node->next;
  }

  return NULL;
}
