/*>=-----------------------------------------------------------------------=<*
 * Vade Mecum: notes module
 * Copyright 2003 by Jamis Buck
 * ------------------------------------------------------------------------
 * This file is part of Vade Mecum.
 *
 * Vade Mecum is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Vade Mecum is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Vade Mecum; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *>=-----------------------------------------------------------------------=<*/

#include "simple-db.h"
#include "notes.h"


typedef struct
{
  SimpleDB db;
}
NoteManagerRec;


NoteManager NoteManager_create( char *filename )
{
  NoteManagerRec *rec;
  SimpleDBDefinition def;

  rec = (NoteManagerRec*)malloc( sizeof( NoteManagerRec ) );

  def.filename = filename;
  def.record_size = 64; /* 64 bytes per "chunk" */
  def.variable_sized_records = TRUE;

  rec->db = SimpleDB_open( &def );

  return (NoteManager)rec;
}


void NoteManager_destroy( NoteManager nm )
{
  NoteManagerRec *rec = (NoteManagerRec*)nm;

  SimpleDB_close( rec->db );
  rec->db = NULL;
  free( rec );
}


Note *NoteManager_getNote( NoteManager nm,
                           int id )
{
  NoteManagerRec *rec = (NoteManagerRec*)nm;
  Note *note;

  note = (Note*)malloc( sizeof( Note ) );
  if( SimpleDB_recordLength( rec->db, id, &note->note_length ) < 0 )
  {
    free( note );
    return NULL;
  }

  note->note = (TCHAR*)malloc( note->note_length );
  note->dbid = id;
  SimpleDB_get( rec->db, id, note->note, note->note_length );

  return note;
}


void NoteManager_releaseNote( Note* note )
{
  free( note->note );
  memset( note, 0, sizeof( *note ) );
  free( note );
}


int NoteManager_addNote( NoteManager nm,
                         TCHAR *note )
{
  NoteManagerRec *rec = (NoteManagerRec*)nm;

  return SimpleDB_insert( rec->db, note, (wcslen(note)+1) * sizeof( TCHAR ) );
}


void NoteManager_deleteNote( NoteManager nm,
                             Note *note )
{
  NoteManagerRec *rec = (NoteManagerRec*)nm;

  NoteManager_deleteNoteByID( nm, note->dbid );
  NoteManager_releaseNote( note );
}


void NoteManager_deleteNoteByID( NoteManager nm,
                                 int id )
{
  NoteManagerRec *rec = (NoteManagerRec*)nm;

  SimpleDB_delete( rec->db, id );
}


void NoteManager_modifyNote( NoteManager nm,
                             Note *note,
                             TCHAR *text )
{
  NoteManagerRec *rec = (NoteManagerRec*)nm;

  if( SimpleDB_update( rec->db, note->dbid, text, ( wcslen(text)+1 ) * sizeof( TCHAR ) ) < 0 )
  {
    return;
  }

  free( note->note );
  note->note = (TCHAR*)malloc( (wcslen(text)+1) * sizeof( TCHAR ) );
  wcscpy( note->note, text );
}
