/*>=-----------------------------------------------------------------------=<*
 * Vade Mecum: reader settings module
 * Copyright 2003 by Jamis Buck
 * Portions copyright 2004 by Henrik Kjoelhede
 * ------------------------------------------------------------------------
 * This file is part of Vade Mecum.
 *
 * Vade Mecum is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Vade Mecum is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Vade Mecum; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *>=-----------------------------------------------------------------------=<*/

#include <windows.h>
#include <commctrl.h>
#include <commdlg.h>
#include <aygshell.h>
#include <dbgapi.h>
#include <stdlib.h>

#include "resrc1.h"
#include "reader-settings.h"
#include "utilities.h"
#include "folder_browser.h"

#define TAB_MARGIN  ( 5 )
#define MIN_FONT_SZ ( 8 )


typedef BOOL (*CALLBACK_FN)( HWND, UINT, WPARAM, LPARAM );
typedef HWND (*CREATE_FN)( HWND, LPRECT );

typedef struct
{
  HWND hWnd;
  int  id;
  CALLBACK_FN callback;
  CREATE_FN constructor;
} REGISTERED_WINDOW;

typedef struct
{
  HWND ctl;
  int family;
} ENUMFONTDATA;


#define  WID_READER_SETTINGS          (  0 )
#define  WID_READER_LIBRARY           (  1 )
#define  WID_READER_FONT              (  2 )
#define  WID_READER_DISPLAY           (  3 )
#define  WID_READER_READER            (  4 )
#define  WID_READER_MISC              (  5 )


#define  TAB_COUNT                    (  5 )

#define  ID_READER_TABS               ( 40001 )
#define  ID_METADATA_PATH             ( 40002 )
#define  ID_BROWSE_METADATA_PATH      ( 40003 )
#define  ID_EBOOK_PATHS               ( 40004 )
#define  ID_ADD_BOOK_PATH             ( 40005 )
#define  ID_DEL_BOOK_PATH             ( 40006 )
#define  ID_REGULAR_FONT              ( 40007 )
#define  ID_MONOSPACE_FONT            ( 40008 )
#define  ID_FONT_SIZE                 ( 40009 )
#define  ID_USE_CLEARTYPE             ( 40010 )
#define  ID_PAGINATION_CACHE_SIZE     ( 40011 )
#define  ID_PARAGRAPH_INDENT          ( 40012 )
#define  ID_JUSTIFICATION_RULE        ( 40013 )
#define  ID_INTERLINE_SPACE           ( 40014 )
#define  ID_INTERPARAGRAPH_SPACE      ( 40015 )
#define  ID_MARGIN_SIZE               ( 40016 )
#define  ID_USE_HSCROLL               ( 40017 )
#define  ID_USE_VSCROLL               ( 40018 )
#define  ID_TOOLBAR_POS               ( 40019 )
#define  ID_SCREEN_DIVISIONS          ( 40020 )
#define  ID_AREA_0                    ( 40021 )
#define  ID_AREA_1                    ( 40022 )
#define  ID_AREA_2                    ( 40023 )
#define  ID_AREA_3                    ( 40024 )
#define  ID_TAP_ACTION                ( 40025 )
#define  ID_FOLLOW_FIRST_LINK         ( 40026 )
#define  ID_FOLLOW_LAST_LINK          ( 40027 )
#define  ID_PROGRESS_METER            ( 40028 )
#define  ID_TOOLBAR_TOGGLE_HOTSPOT    ( 40029 )
#define  ID_FIRST_PARAGRAPH_IS_TITLE  ( 40030 )
#define  ID_ENABLE_SOFT_HYPHENS       ( 40031 )


/* instruct a page to save it's contents */
#define WM_SAVE_DATA                  ( WM_USER+1 )


#define EF_REGULAR    ( 0 )
#define EF_MONOSPACE  ( 1 )


static Library *g_settings;
static UINT32   g_settings_changed;
static UINT16   g_screen_division_top;
static UINT8    g_screen_division;
static UINT8    g_tap_actions[4];
static UINT8    g_active_area;
static RECT     g_item_rect;  /* the bounding rectangle for the property sheets */


static REGISTERED_WINDOW g_hwnd[ TAB_COUNT+1 ] = { 0 }; /* '0' is the window itself */


static void DoInitReaderSettingsDlg( HWND hwndDlg );
static void DoInitLibrarySettings( HWND hWnd );
static void DoInitFontSettings( HWND hWnd );
static void DoInitDisplaySettings( HWND hWnd );
static void DoInitReaderSettings( HWND hWnd );
static void DoInitMiscSettings( HWND hWnd );

static HWND CreateReaderSettingsDlg( HWND hParent );
static HWND CreateLibrarySettingsPage( HWND hParent, LPRECT rect );
static HWND CreateFontSettingsPage( HWND hParent, LPRECT rect );
static HWND CreateDisplaySettingsPage( HWND hParent, LPRECT rect );
static HWND CreateReaderSettingsPage( HWND hParent, LPRECT rect );
static HWND CreateMiscSettingsPage( HWND hParent, LPRECT rect );

static BOOL MainReaderSettingsProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );
static BOOL LibrarySettingsProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );
static BOOL FontSettingsProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );
static BOOL DisplaySettingsProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );
static BOOL ReaderSettingsProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );
static BOOL MiscSettingsProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );


static void RegisterReaderSettingWindow( int id, CALLBACK_FN callback, CREATE_FN constructor );

static BOOL DoReaderNotify( HWND hWnd, WPARAM wParam, LPARAM lParam );
static void ShowTab( HWND hWnd, int which, BOOL show );
static void SaveAll();

static void DoBrowseMetadataPath( HWND hWnd );
static void DoAddBookPath( HWND hWnd );
static void DoDelBookPath( HWND hWnd );

static void DoSaveLibrarySettings( HWND hWnd );
static void DoSaveDisplaySettings( HWND hWnd );
static void DoSaveFontSettings( HWND hWnd );
static void DoSaveReaderSettings( HWND hWnd );
static void DoSaveMiscSettings( HWND hWnd );

static void EnumerateFontsIntoList( HWND hCtl, int family );
static void SelectThis( HWND hCtl, TCHAR* str );

static int CALLBACK FontEnumCallback( CONST ENUMLOGFONT FAR *lpelf,
                                      CONST TEXTMETRIC FAR *lpntm,
                                      DWORD FontType,
                                      LPARAM lParam );

static void DrawScreenDivision( HWND hWnd, HDC dc );
static void UpdateTapActionList( HWND hWnd, int area );


UINT32 OpenReaderSettings( HWND hWnd, Library *settings )
{
  MSG msg;
  HWND hReader;
  SHMENUBARINFO mbi;

  g_settings = settings;
  g_settings_changed = 0;

  RegisterReaderSettingWindow( WID_READER_SETTINGS, MainReaderSettingsProc, NULL );
  RegisterReaderSettingWindow( WID_READER_LIBRARY, LibrarySettingsProc, CreateLibrarySettingsPage );
  RegisterReaderSettingWindow( WID_READER_FONT, FontSettingsProc, CreateFontSettingsPage );
  RegisterReaderSettingWindow( WID_READER_DISPLAY, DisplaySettingsProc, CreateDisplaySettingsPage );
  RegisterReaderSettingWindow( WID_READER_READER, ReaderSettingsProc, CreateReaderSettingsPage );
  RegisterReaderSettingWindow( WID_READER_MISC, MiscSettingsProc, CreateMiscSettingsPage );

  hReader = CreateReaderSettingsDlg( hWnd );

  mbi.cbSize = sizeof( mbi );
  mbi.hwndParent = hReader;
  mbi.dwFlags = SHCMBF_EMPTYBAR;
  mbi.hInstRes = settings->hInstance;

  SHFullScreen( hReader, SHFS_SHOWSIPBUTTON );
  SHCreateMenuBar( &mbi );

  ShowWindow( hReader, SW_SHOW );
  UpdateWindow( hReader );

  while( GetMessage( &msg, NULL, 0, 0 ) ) 
  {
    TranslateMessage( &msg );
    DispatchMessage( &msg );
  }

  return g_settings_changed;
}


BOOL CALLBACK ReaderSettingsDialogProc( HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
  int i;
  LPCREATESTRUCT cs;

  if( uMsg == WM_CREATE )
  {
    cs = (LPCREATESTRUCT)lParam;
    for( i = 0; i <= TAB_COUNT; i++ )
    {
      if( (LONG)cs->lpCreateParams == g_hwnd[ i ].id )
      {
        g_hwnd[ i ].hWnd = hwndDlg;
        return g_hwnd[ i ].callback( hwndDlg, uMsg, wParam, lParam );
      }
    }
  }
  else
  {
    for( i = 0; i <= TAB_COUNT; i++ )
    {
      if( hwndDlg == g_hwnd[ i ].hWnd )
      {
        return g_hwnd[ i ].callback( hwndDlg, uMsg, wParam, lParam );
      }
    }
  }

  return FALSE;
}


static void DoInitReaderSettingsDlg( HWND hwndDlg )
{
  SHINITDLGINFO shidi;
  HWND ctl;
  RECT rect;
  TCITEM item;

  shidi.dwMask = SHIDIM_FLAGS;
  shidi.hDlg = hwndDlg;
  shidi.dwFlags = SHIDIF_DONEBUTTON | SHIDIF_SIZEDLG;
  SHInitDialog( &shidi );

  GetClientRect( hwndDlg, &rect );

  ctl = CreateWindow( WC_TABCONTROL, L"", WS_CHILD | WS_BORDER | WS_VISIBLE |
                        TCS_TABS | TCS_SINGLELINE | TCS_RAGGEDRIGHT,
                      0, 0, rect.right - rect.left, rect.bottom - rect.top,
                      hwndDlg, (HMENU)ID_READER_TABS, g_settings->hInstance, NULL );

  item.mask = TCIF_TEXT;
  item.pszText = L"Library";
  TabCtrl_InsertItem( ctl, 0, &item );

  item.pszText = L"Font";
  TabCtrl_InsertItem( ctl, 1, &item );

  item.pszText = L"Display";
  TabCtrl_InsertItem( ctl, 2, &item );

  item.pszText = L"Reader";
  TabCtrl_InsertItem( ctl, 3, &item );

  item.pszText = L"Misc";
  TabCtrl_InsertItem( ctl, 4, &item );

  memcpy( &g_item_rect, &rect, sizeof( RECT ) );
  TabCtrl_AdjustRect( ctl, FALSE, &g_item_rect );

  ShowTab( hwndDlg, 0, TRUE );
}


static HWND CreateReaderSettingsDlg( HWND hParent )
{
  HWND hWnd;
  TCHAR szWindowClass[ 64 ];

  LoadString( g_settings->hInstance, IDS_READER_SETTINGS_CLASS, szWindowClass, 64 );
  hWnd = CreateWindow( szWindowClass, L"Reader Settings", WS_VISIBLE | WS_POPUP,
                       CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
                       hParent, (HMENU)NULL, g_settings->hInstance, (LPVOID)WID_READER_SETTINGS );

  return hWnd;
}


static HWND CreateLibrarySettingsPage( HWND hParent, LPRECT rect )
{
  HWND hWnd;
  TCHAR szWindowClass[ 64 ];

  LoadString( g_settings->hInstance, IDS_READER_SETTINGS_CLASS, szWindowClass, 64 );
  hWnd = CreateWindow( szWindowClass, L"", WS_CHILD,
                       rect->left, rect->top, rect->right - rect->left, rect->bottom - rect->top,
                       hParent, (HMENU)NULL, g_settings->hInstance, (LPVOID)WID_READER_LIBRARY );

  return hWnd;
}


static HWND CreateFontSettingsPage( HWND hParent, LPRECT rect )
{
  HWND hWnd;
  TCHAR szWindowClass[ 64 ];

  LoadString( g_settings->hInstance, IDS_READER_SETTINGS_CLASS, szWindowClass, 64 );
  hWnd = CreateWindow( szWindowClass, L"", WS_CHILD,
                       rect->left, rect->top, rect->right - rect->left, rect->bottom - rect->top,
                       hParent, (HMENU)NULL, g_settings->hInstance, (LPVOID)WID_READER_FONT );

  return hWnd;
}


static BOOL MainReaderSettingsProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
  switch( uMsg )
  {
    case WM_CREATE:
      DoInitReaderSettingsDlg( hWnd );
      break;

    case WM_COMMAND:
      switch( LOWORD( wParam ) )
      {
        case IDOK:
          SHFullScreen( hWnd, SHFS_HIDESIPBUTTON );
          SaveAll();
          DestroyWindow( hWnd );
          break;

        case IDCANCEL:
          SHFullScreen( hWnd, SHFS_HIDESIPBUTTON );
          DestroyWindow( hWnd );
          break;

        default:
          return FALSE;
      }
      break;

    case WM_NOTIFY:
      return DoReaderNotify( hWnd, wParam, lParam );

    case WM_CLOSE:
      SHFullScreen( hWnd, SHFS_HIDESIPBUTTON );
      DestroyWindow( hWnd );
      break;

    case WM_DESTROY:
      PostQuitMessage( 0 );
      break;

    default:
      return DefWindowProc( hWnd, uMsg, wParam, lParam );
  }

  return TRUE;
}


static BOOL LibrarySettingsProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
  switch( uMsg )
  {
    case WM_CREATE:
      DoInitLibrarySettings( hWnd );
      break;

    case WM_COMMAND:
      switch( LOWORD( wParam ) )
      {
        case ID_BROWSE_METADATA_PATH:
          DoBrowseMetadataPath( hWnd );
          break;

        case ID_ADD_BOOK_PATH:
          DoAddBookPath( hWnd );
          break;

        case ID_DEL_BOOK_PATH:
          DoDelBookPath( hWnd );
          break;

        default:
          return FALSE;
      }
      break;

    case WM_SAVE_DATA:
      DoSaveLibrarySettings( hWnd );
      break;

    default:
      return DefWindowProc( hWnd, uMsg, wParam, lParam );
  }

  return TRUE;
}


static BOOL DisplaySettingsProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
  switch( uMsg )
  {
    case WM_CREATE:
      DoInitDisplaySettings( hWnd );
      break;

    case WM_SAVE_DATA:
      DoSaveDisplaySettings( hWnd );
      break;

    default:
      return DefWindowProc( hWnd, uMsg, wParam, lParam );
  }

  return TRUE;
}


static BOOL FontSettingsProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
  switch( uMsg )
  {
    case WM_CREATE:
      DoInitFontSettings( hWnd );
      break;

    case WM_SAVE_DATA:
      DoSaveFontSettings( hWnd );
      break;

    default:
      return DefWindowProc( hWnd, uMsg, wParam, lParam );
  }

  return TRUE;
}


static void RegisterReaderSettingWindow( int id, CALLBACK_FN callback, CREATE_FN constructor )
{
  g_hwnd[ id ].id = id;
  g_hwnd[ id ].callback = callback;
  g_hwnd[ id ].constructor = constructor;
  g_hwnd[ id ].hWnd = NULL;
}


static void DoInitLibrarySettings( HWND hWnd )
{
  RECT rect;
  HWND ctl;
  int xofs;
  int yofs;
  int width;
  int height;
  int i, j;
  TCHAR buf[ MAX_PATH ];

  /* meta-data path (metadata.vdm)
   * ebook-paths (use OFN_PROJECT in GetOpenFileName to open Folder dialog box) */

  GetClientRect( hWnd, &rect );

  yofs = TAB_MARGIN;
  height = 50;

  CreateStaticControl( hWnd, g_settings->hInstance, 0,
                       L"This tab allows you to select the location of the library's "
                       L"metadata as well as where your ebooks are stored.",
                       TAB_MARGIN, yofs,
                       rect.right - rect.left - 2*TAB_MARGIN,
                       height, SS_LEFT | WS_VISIBLE );

  yofs += height + 5;
  height = 20;

  xofs = TAB_MARGIN;
  width = 55;

  CreateStaticControl( hWnd, g_settings->hInstance, 0,
                       L"Metadata:",
                       xofs, yofs+4,
                       width,
                       height-4, SS_LEFTNOWORDWRAP | WS_VISIBLE );

  xofs += width + 5;
  width = rect.right - rect.left - 31 - xofs - TAB_MARGIN;

  ctl = CreateEditControl( hWnd, g_settings->hInstance, ID_METADATA_PATH,
                           xofs, yofs,
                           width,
                           height, ES_LEFT | WS_VISIBLE );
  SetWindowText( ctl, g_settings->metadata_path );

  xofs += width + 5;
  width = 26;
  CreateButtonControl( hWnd, g_settings->hInstance, ID_BROWSE_METADATA_PATH, L"...",
                       xofs, yofs,
                       width, height, WS_VISIBLE, FALSE );

  yofs += height + 8;
  height = 16;
  xofs = TAB_MARGIN;
  width = 100;

  CreateStaticControl( hWnd, g_settings->hInstance, 0,
                       L"Document Paths:",
                       xofs, yofs,
                       width, height,
                       SS_LEFTNOWORDWRAP | WS_VISIBLE );

  yofs += height + 2;
  height = rect.bottom - rect.top - yofs - 35;
  xofs = TAB_MARGIN;
  width = rect.right - rect.left - 2*TAB_MARGIN - 5 - 30;

  ctl = CreateListBox( hWnd, g_settings->hInstance, ID_EBOOK_PATHS,
                       xofs, yofs, width, height, WS_VSCROLL | WS_VISIBLE | LBS_NOINTEGRALHEIGHT );

  xofs += width + 5;
  height = height/2;
  width = 30;

  CreateButtonControl( hWnd, g_settings->hInstance, ID_ADD_BOOK_PATH, L"Add",
                       xofs, yofs, width, height, WS_VISIBLE, FALSE );
  
  yofs += height;

  CreateButtonControl( hWnd, g_settings->hInstance, ID_DEL_BOOK_PATH, L"Del",
                       xofs, yofs, width, height, WS_VISIBLE, FALSE );

  /* populate the list with the contents of the (semicolon-delimited) ebook path list */

  i = 0;
  j = 0;
  while( g_settings->ebook_paths[i] != 0 )
  {
    if( g_settings->ebook_paths[i] == ';' )
    {
      buf[j] = 0;
      SendMessage( ctl, LB_ADDSTRING, 0, (LPARAM)buf );
      j = 0;
    }
    else
    {
      buf[j] = g_settings->ebook_paths[i];
      j++;
    }
    i++;
  }

  if( j > 0 )
  {
    buf[j] = 0;
    SendMessage( ctl, LB_ADDSTRING, 0, (LPARAM)buf );
  }

  yofs += height + 5;
  xofs = TAB_MARGIN;
  height = 20;
  width = 160;

  CreateStaticControl( hWnd, g_settings->hInstance, 0, L"Cache size (per document):",
                       xofs, yofs+4, width, height-4, SS_LEFT | WS_VISIBLE );
  xofs += width + 5;

  width = 40;
  ctl = CreateComboBox( hWnd, g_settings->hInstance, ID_PAGINATION_CACHE_SIZE, xofs, yofs, width, height*5, 
                        CBS_DROPDOWNLIST | WS_VISIBLE | WS_VSCROLL );

  for( i = 1; i <= 10; i++ )
  {
    wsprintf( buf, L"%d", i );
    SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)buf );
  }

  SendMessage( ctl, CB_SETCURSEL, g_settings->pagination_cache_size-1, 0 );
}


static BOOL DoReaderNotify( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
  NMHDR *hdr = (NMHDR*)lParam;
  int sel;

  switch( wParam )
  {
    case ID_READER_TABS:
     switch( hdr->code )
      {
        case TCN_SELCHANGING:
          sel = TabCtrl_GetCurSel( hdr->hwndFrom );
          ShowTab( hWnd, sel, FALSE );
          return FALSE;

        case TCN_SELCHANGE:
          sel = TabCtrl_GetCurSel( hdr->hwndFrom );
          ShowTab( hWnd, sel, TRUE );
          break;

        default:
          return DefWindowProc( hWnd, WM_NOTIFY, wParam, lParam );
      }
      break;

    default:
      return DefWindowProc( hWnd, WM_NOTIFY, wParam, lParam );
  }

  return FALSE;
}


static void ShowTab( HWND hWnd, int which, BOOL show )
{
  if( g_hwnd[ which+1 ].constructor != NULL )
  {
    if( g_hwnd[ which+1 ].hWnd == NULL )
    {
      g_hwnd[ which+1 ].hWnd = g_hwnd[ which+1 ].constructor( hWnd, &g_item_rect );
    }

    if( g_hwnd[ which+1 ].hWnd != NULL )
    {
      ShowWindow( g_hwnd[ which+1 ].hWnd, ( show ? SW_SHOW : SW_HIDE ) );
      EnableWindow( g_hwnd[ which+1 ].hWnd, show );

      if( show )
      {
        SetWindowPos( g_hwnd[ which+1 ].hWnd, HWND_TOP, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_SHOWWINDOW );
      }
    
      UpdateWindow( g_hwnd[ which+1 ].hWnd );
    }
  }
}


static void DoBrowseMetadataPath( HWND hWnd )
{
  FOLDERBROWSERINFO fbi;
  TCHAR folder[ MAX_PATH ];

  memset( &fbi, 0, sizeof( fbi ) );
  folder[0] = 0;

  fbi.folder_path = folder;
  fbi.folder_path_len = MAX_PATH;
  fbi.hInstance = g_settings->hInstance;
  fbi.hParent = hWnd;
  fbi.title = L"Metadata Folder";

  if( GetFolderName( &fbi ) )
  {
    HWND hMetadata;

    hMetadata = GetDlgItem( hWnd, ID_METADATA_PATH );
    SetWindowText( hMetadata, fbi.folder_path );
  }
}


static void DoAddBookPath( HWND hWnd )
{
  FOLDERBROWSERINFO fbi;
  TCHAR folder[ MAX_PATH ];

  memset( &fbi, 0, sizeof( fbi ) );
  folder[0] = 0;

  fbi.folder_path = folder;
  fbi.folder_path_len = MAX_PATH;
  fbi.hInstance = g_settings->hInstance;
  fbi.hParent = hWnd;
  fbi.title = L"Document Folder";

  if( GetFolderName( &fbi ) )
  {
    HWND hList;

    hList = GetDlgItem( hWnd, ID_EBOOK_PATHS );

    /* don't allow the same string to be added twice */
    if( SendMessage( hList, LB_FINDSTRINGEXACT, 0, (LPARAM)fbi.folder_path ) == LB_ERR )
    {
      SendMessage( hList, LB_ADDSTRING, 0, (LPARAM)fbi.folder_path );
    }
  }
}


static void DoDelBookPath( HWND hWnd )
{
  HWND hList;
  int sel;

  hList = GetDlgItem( hWnd, ID_EBOOK_PATHS );
  sel = SendMessage( hList, LB_GETCURSEL, 0, 0 );

  if( sel == LB_ERR )
  {
    return;
  }

  SendMessage( hList, LB_DELETESTRING, sel, 0 );
}


static void DoSaveLibrarySettings( HWND hWnd )
{
  HWND hctl;
  int i;
  int count;
  TCHAR item[MAX_PATH];
  TCHAR paths[MAX_EBOOK_PATHS_LEN];

  /* first, save the metadata path */
  hctl = GetDlgItem( hWnd, ID_METADATA_PATH );
  GetWindowText( hctl, item, MAX_PATH );

  if( wcscmp( item, g_settings->metadata_path ) != 0 )
  {
    g_settings_changed |= RS_METADATA_CHANGED;
    wcscpy( g_settings->metadata_path, item );
  }

  /* next, save the list of ebook paths */
  hctl = GetDlgItem( hWnd, ID_EBOOK_PATHS );
  count = SendMessage( hctl, LB_GETCOUNT, 0, 0 );

  paths[0] = 0;
  for( i = 0; i < count; i++ )
  {
    if( i > 0 ) wcscat( paths, L";" );
    SendMessage( hctl, LB_GETTEXT, i, (LPARAM)item );
    wcscat( paths, item );
  }

  if( wcscmp( paths, g_settings->ebook_paths ) != 0 )
  {
    g_settings_changed |= RS_BOOKPATH_CHANGED;
    wcscpy( g_settings->ebook_paths, paths );
  }

  hctl = GetDlgItem( hWnd, ID_PAGINATION_CACHE_SIZE );
  i = SendMessage( hctl, CB_GETCURSEL, 0, 0 );
  if( i+1 != g_settings->pagination_cache_size )
  {
    g_settings->pagination_cache_size = i+1;
    g_settings_changed |= RS_METADATA_CHANGED;
  }
}


static void SaveAll()
{
  int i;

  for( i = 1; i <= TAB_COUNT; i++ )
  {
    if( g_hwnd[i].hWnd != NULL )
    {
      SendMessage( g_hwnd[i].hWnd, WM_SAVE_DATA, 0, 0 );
    }
  }
}


static void DoInitFontSettings( HWND hWnd )
{
  RECT rect;
  HWND ctl;
  int xofs;
  int yofs;
  int width;
  int height;
  int i;

  GetClientRect( hWnd, &rect );

  yofs = TAB_MARGIN;
  height = 35;

  CreateStaticControl( hWnd, g_settings->hInstance, 0,
                       L"Here you can alter the font settings for the reader.",
                       TAB_MARGIN, yofs,
                       rect.right - rect.left - 2*TAB_MARGIN,
                       height, SS_LEFT | WS_VISIBLE );

  /* normal-font: [dropdown -- serif/sans-serif]
   * monospace-font: [dropdown -- monospace]
   * font-size: [dropdown -- 6--14]
   * use-cleartype [checkbox]
   */

  yofs += height;
  xofs = TAB_MARGIN;
  width = 100;
  height = 20;

  CreateStaticControl( hWnd, g_settings->hInstance, 0,
                       L"Regular Font:",
                       xofs, yofs+3, width, height-3, SS_LEFTNOWORDWRAP | WS_VISIBLE );

  xofs += width + 5;
  width = rect.right - TAB_MARGIN - xofs;

  ctl = CreateComboBox( hWnd, g_settings->hInstance, ID_REGULAR_FONT,
                        xofs, yofs, width, height*4, CBS_DROPDOWNLIST | CBS_SORT | WS_VISIBLE | WS_VSCROLL );
  EnumerateFontsIntoList( ctl, EF_REGULAR );
  SelectThis( ctl, g_settings->options.normal_face );

  yofs += height + 3;
  xofs = TAB_MARGIN;
  width = 100;
  height = 20;

  CreateStaticControl( hWnd, g_settings->hInstance, 0,
                       L"Monospace Font:",
                       xofs, yofs+3, width, height-3, SS_LEFTNOWORDWRAP | WS_VISIBLE );

  xofs += width + 5;
  width = rect.right - TAB_MARGIN - xofs;

  ctl = CreateComboBox( hWnd, g_settings->hInstance, ID_MONOSPACE_FONT,
                        xofs, yofs, width, height*4, CBS_DROPDOWNLIST | CBS_SORT | WS_VISIBLE | WS_VSCROLL );
  EnumerateFontsIntoList( ctl, EF_MONOSPACE );
  SelectThis( ctl, g_settings->options.monotype_face );

  yofs += height + 6;
  xofs = TAB_MARGIN;
  width = 100;
  height = 20;

  CreateStaticControl( hWnd, g_settings->hInstance, 0,
                       L"Font Size:",
                       xofs, yofs+3, width, height-3, SS_LEFTNOWORDWRAP | WS_VISIBLE );

  xofs += width + 5;
  width = ( rect.right - TAB_MARGIN - xofs ) / 2;

  ctl = CreateComboBox( hWnd, g_settings->hInstance, ID_FONT_SIZE,
                        xofs, yofs, width, height*4, CBS_DROPDOWNLIST | WS_VISIBLE | WS_VSCROLL );

  for( i = MIN_FONT_SZ; i <= 16; i++ )
  {
    TCHAR item[3];

    wsprintf( item, L"%d", i );
    SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)item );
  }

  SendMessage( ctl, CB_SETCURSEL, g_settings->options.font_size-MIN_FONT_SZ, 0 );

  yofs += height + 10;
  xofs = TAB_MARGIN;
  height = 20;
  width = 100;
  ctl = CreateCheckBox( hWnd, g_settings->hInstance, ID_USE_CLEARTYPE, L"Use ClearType",
                        xofs, yofs, width, height, WS_VISIBLE );

  if( g_settings->options.text_quality == USE_CLEARTYPE_QUALITY )
  {
    SendMessage( ctl, BM_SETCHECK, BST_CHECKED, 0 );
  }
}


static void DoSaveFontSettings( HWND hWnd )
{
  TCHAR item[64];
  int sel;
  int quality;
  HWND ctl;

  /* get the selected "regular" font */
  ctl = GetDlgItem( hWnd, ID_REGULAR_FONT );
  sel = SendMessage( ctl, CB_GETCURSEL, 0, 0 );
  SendMessage( ctl, CB_GETLBTEXT, sel, (LPARAM)item );
  if( _wcsicmp( item, g_settings->options.normal_face ) != 0 )
  {
    g_settings_changed |= RS_DISPLAY_PARMS_CHANGED;
    wcscpy( g_settings->options.normal_face, item );
  }

  /* get the selected "monospace" font */
  ctl = GetDlgItem( hWnd, ID_MONOSPACE_FONT );
  sel = SendMessage( ctl, CB_GETCURSEL, 0, 0 );
  SendMessage( ctl, CB_GETLBTEXT, sel, (LPARAM)item );
  if( _wcsicmp( item, g_settings->options.monotype_face ) != 0 )
  {
    g_settings_changed |= RS_DISPLAY_PARMS_CHANGED;
    wcscpy( g_settings->options.monotype_face, item );
  }

  /* get the font size */
  ctl = GetDlgItem( hWnd, ID_FONT_SIZE );
  sel = SendMessage( ctl, CB_GETCURSEL, 0, 0 );
  if( sel+MIN_FONT_SZ != g_settings->options.font_size )
  {
    g_settings_changed |= RS_DISPLAY_PARMS_CHANGED;
    g_settings->options.font_size = sel+MIN_FONT_SZ;
  }

  /* examine the cleartype selection */
  ctl = GetDlgItem( hWnd, ID_USE_CLEARTYPE );
  quality = ( ( SendMessage( ctl, BM_GETCHECK, 0, 0 ) == BST_CHECKED ) ? USE_CLEARTYPE_QUALITY : DEFAULT_QUALITY );
  if( quality != g_settings->options.text_quality )
  {
    g_settings_changed |= RS_DISPLAY_PARMS_CHANGED;
    g_settings->options.text_quality = quality;
  }
}


static void EnumerateFontsIntoList( HWND hCtl, int family )
{
  ENUMFONTDATA data;
  HDC dc;

  data.ctl = hCtl;
  data.family = family;

  dc = GetDC( hCtl );
  EnumFontFamilies( dc, NULL, (FONTENUMPROC)FontEnumCallback, (LPARAM)&data );
  ReleaseDC( hCtl, dc );
}


static int CALLBACK FontEnumCallback( CONST ENUMLOGFONT FAR *lpelf,
                                      CONST TEXTMETRIC FAR *lpntm,
                                      DWORD FontType,
                                      LPARAM lParam )
{
  ENUMFONTDATA *data = (ENUMFONTDATA*)lParam;
  int fam;

  fam = ( lpelf->elfLogFont.lfPitchAndFamily & 0xFC );

  if( ( ( data->family == EF_REGULAR ) && ( ( fam == FF_SWISS ) || ( fam == FF_ROMAN ) ) ) ||
      ( ( data->family == EF_MONOSPACE ) && ( ( fam == FF_MODERN ) ) ) )
  {
    SendMessage( data->ctl, CB_ADDSTRING, 0, (LPARAM)lpelf->elfFullName );
  }
  return 1;
}


static void SelectThis( HWND hCtl, TCHAR* str )
{
  int count;
  int i;
  TCHAR item[64];

  count = SendMessage( hCtl, CB_GETCOUNT, 0, 0 );
  for( i = 0; i < count; i++ )
  {
    SendMessage( hCtl, CB_GETLBTEXT, i, (LPARAM)item );
    if( _wcsicmp( item, str ) == 0 )
    {
      SendMessage( hCtl, CB_SETCURSEL, i, 0 );
      break;
    }
  }
}


static HWND CreateDisplaySettingsPage( HWND hParent, LPRECT rect )
{
  HWND hWnd;
  TCHAR szWindowClass[ 64 ];

  LoadString( g_settings->hInstance, IDS_READER_SETTINGS_CLASS, szWindowClass, 64 );
  hWnd = CreateWindow( szWindowClass, L"", WS_CHILD,
                       rect->left, rect->top, rect->right - rect->left, rect->bottom - rect->top,
                       hParent, (HMENU)NULL, g_settings->hInstance, (LPVOID)WID_READER_DISPLAY );

  return hWnd;
}


static void DoInitDisplaySettings( HWND hWnd )
{
  RECT rect;
  HWND ctl;
  int xofs;
  int yofs;
  int width;
  int height;
  int i;
  TCHAR value[64];

  GetClientRect( hWnd, &rect );

  yofs = TAB_MARGIN;
  height = 35;

  CreateStaticControl( hWnd, g_settings->hInstance, 0,
                       L"Here you can alter the display settings for the reader.",
                       TAB_MARGIN, yofs,
                       rect.right - rect.left - 2*TAB_MARGIN,
                       height, SS_LEFT | WS_VISIBLE );

  yofs += height;
  xofs = TAB_MARGIN;
  width = 60;
  height = 20;

  CreateStaticControl( hWnd, g_settings->hInstance, 0, L"Scrollbars:", xofs, yofs+4, width, height-4, SS_LEFT | WS_VISIBLE );

  xofs += width + 5;
  width = 80;

  ctl = CreateCheckBox( hWnd, g_settings->hInstance, ID_USE_HSCROLL, L"Horizontal",
                        xofs, yofs, width, height, WS_VISIBLE | WS_DISABLED);

  if( g_settings->use_hscroll )
  {
    SendMessage( ctl, BM_SETCHECK, BST_CHECKED, 0 );
  }

  xofs += width + 5;

  ctl = CreateCheckBox( hWnd, g_settings->hInstance, ID_USE_VSCROLL, L"Vertical",
                        xofs, yofs, width, height, WS_VISIBLE );

  if( g_settings->use_vscroll )
  {
    SendMessage( ctl, BM_SETCHECK, BST_CHECKED, 0 );
  }

  yofs += height;
  xofs = TAB_MARGIN;
  width = 100;
  height = 20;

  CreateStaticControl( hWnd, g_settings->hInstance, 0, L"Margins:", xofs, yofs+4, width, height-6, SS_LEFT | WS_VISIBLE );

  xofs += width + 5;
  width = 60;

  wsprintf( value, L"%d", g_settings->options.left_margin );
  ctl = CreateEditControl( hWnd, g_settings->hInstance, ID_MARGIN_SIZE, xofs, yofs, width, height, ES_LEFT | WS_VISIBLE );
  SetWindowText( ctl, value );

  yofs += height + 5;
  xofs = TAB_MARGIN;
  width = 100;

  CreateStaticControl( hWnd, g_settings->hInstance, 0, L"Indentation:", xofs, yofs+4, width, height-6, SS_LEFT | WS_VISIBLE );

  xofs += width + 5;
  width = 60;

  wsprintf( value, L"%d", g_settings->options.indent_size );
  ctl = CreateEditControl( hWnd, g_settings->hInstance, ID_PARAGRAPH_INDENT, xofs, yofs, width, height, ES_LEFT | WS_VISIBLE );
  SetWindowText( ctl, value );

  yofs += height + 5;
  xofs = TAB_MARGIN;
  width = 100;

  CreateStaticControl( hWnd, g_settings->hInstance, 0, L"Interline Space:", xofs, yofs+4, width, height-6, SS_LEFT | WS_VISIBLE );

  xofs += width + 5;
  width = 60;

  wsprintf( value, L"%d", g_settings->options.interline_space );
  ctl = CreateEditControl( hWnd, g_settings->hInstance, ID_INTERLINE_SPACE, xofs, yofs, width, height, ES_LEFT | WS_VISIBLE );
  SetWindowText( ctl, value );

  yofs += height + 5;
  xofs = TAB_MARGIN;
  width = 100;

  CreateStaticControl( hWnd, g_settings->hInstance, 0, L"Interparagraph:", xofs, yofs+4, width, height-6, SS_LEFT | WS_VISIBLE );

  xofs += width + 5;
  width = 60;

  wsprintf( value, L"%d", g_settings->options.interparagraph_space );
  ctl = CreateEditControl( hWnd, g_settings->hInstance, ID_INTERPARAGRAPH_SPACE, xofs, yofs, width, height, ES_LEFT | WS_VISIBLE );
  SetWindowText( ctl, value );

  yofs += height + 5;
  xofs = TAB_MARGIN;
  width = 100;

  CreateStaticControl( hWnd, g_settings->hInstance, 0, L"Justification:", xofs, yofs+4, width, height-6, SS_LEFT | WS_VISIBLE );

  xofs += width + 5;
  width = 60;

  ctl = CreateComboBox( hWnd, g_settings->hInstance, ID_JUSTIFICATION_RULE, xofs, yofs, width, height*4,
                        CBS_DROPDOWNLIST | WS_VISIBLE | WS_VSCROLL );

  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"Normal" );
  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"Always" );
  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"Never" );

  switch( g_settings->options.justification_rule )
  {
    case JUSTIFY_NORMAL: i = 0; break;
    case JUSTIFY_ALWAYS: i = 1; break;
    case JUSTIFY_NEVER:  i = 2; break;
  }

  SendMessage( ctl, CB_SETCURSEL, i, 0 );
}


static void DoSaveDisplaySettings( HWND hWnd )
{
  TCHAR value[64];
  HWND  ctl;
  int   i;
  TCHAR *ep;
  BOOL  checked;

  ctl = GetDlgItem( hWnd, ID_USE_HSCROLL );
  checked = ( SendMessage( ctl, BM_GETCHECK, 0, 0 ) == BST_CHECKED );

  if( checked != g_settings->use_hscroll )
  {
    g_settings_changed |= RS_DISPLAY_PARMS_CHANGED;
    g_settings->use_hscroll = checked;
  }

  ctl = GetDlgItem( hWnd, ID_USE_VSCROLL );
  checked = ( SendMessage( ctl, BM_GETCHECK, 0, 0 ) == BST_CHECKED );

  if( checked != g_settings->use_vscroll )
  {
    g_settings_changed |= RS_DISPLAY_PARMS_CHANGED;
    g_settings->use_vscroll = checked;
  }

  ctl = GetDlgItem( hWnd, ID_MARGIN_SIZE );
  GetWindowText( ctl, value, 64 );
  i = wcstol( value, &ep, 10 );

  if( (int)g_settings->options.left_margin != i || (int)g_settings->options.right_margin != i )
  {
    g_settings->options.left_margin = g_settings->options.right_margin = i;
    g_settings_changed |= RS_DISPLAY_PARMS_CHANGED;
  }

  ctl = GetDlgItem( hWnd, ID_PARAGRAPH_INDENT );
  GetWindowText( ctl, value, 64 );
  i = wcstol( value, &ep, 10 );

  if( g_settings->options.indent_size != i )
  {
    g_settings->options.indent_size = i;
    g_settings_changed |= RS_DISPLAY_PARMS_CHANGED;
  }

  ctl = GetDlgItem( hWnd, ID_INTERLINE_SPACE );
  GetWindowText( ctl, value, 64 );
  i = wcstol( value, &ep, 10 );

  if( g_settings->options.interline_space != i )
  {
    g_settings->options.interline_space = i;
    g_settings_changed |= RS_DISPLAY_PARMS_CHANGED;
  }

  ctl = GetDlgItem( hWnd, ID_INTERPARAGRAPH_SPACE );
  GetWindowText( ctl, value, 64 );
  i = wcstol( value, &ep, 10 );

  if( g_settings->options.interparagraph_space != i )
  {
    g_settings->options.interparagraph_space = i;
    g_settings_changed |= RS_DISPLAY_PARMS_CHANGED;
  }

  ctl = GetDlgItem( hWnd, ID_JUSTIFICATION_RULE );
  i = SendMessage( ctl, CB_GETCURSEL, 0, 0 );

  switch( i )
  {
    case 0: i = JUSTIFY_NORMAL; break;
    case 1: i = JUSTIFY_ALWAYS; break;
    case 2: i = JUSTIFY_NEVER; break;
  }

  if( g_settings->options.justification_rule != i )
  {
    g_settings->options.justification_rule = i;
    g_settings_changed |= RS_DISPLAY_PARMS_CHANGED;
  }
}


static BOOL ReaderSettingsProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
  switch( uMsg )
  {
    case WM_CREATE:
      DoInitReaderSettings( hWnd );
      break;

    case WM_SAVE_DATA:
      DoSaveReaderSettings( hWnd );
      break;

    case WM_PAINT:
      DefWindowProc( hWnd, uMsg, wParam, lParam );
      DrawScreenDivision( hWnd, NULL );
      break;

    case WM_ACTIVATE:
      DrawScreenDivision( hWnd, NULL );
      break;

    case WM_COMMAND:
      switch( LOWORD( wParam ) )
      {
        case ID_SCREEN_DIVISIONS:
          switch( HIWORD( wParam ) )
          {
            case CBN_SELENDOK:
              g_screen_division = (unsigned char)SendMessage( (HWND)lParam, CB_GETCURSEL, 0, 0 );
              InvalidateRect( hWnd, NULL, FALSE );
              break;
          }
          break;

        case ID_AREA_0:
          UpdateTapActionList( hWnd, 0 );
          break;

        case ID_AREA_1:
          UpdateTapActionList( hWnd, 1 );
          break;

        case ID_AREA_2:
          UpdateTapActionList( hWnd, 2 );
          break;

        case ID_AREA_3:
          UpdateTapActionList( hWnd, 3 );
          break;

        case ID_TAP_ACTION:
          switch( HIWORD( wParam ) )
          {
            case CBN_SELENDOK:
              g_tap_actions[ g_active_area ] = (unsigned char)SendMessage( (HWND)lParam, CB_GETCURSEL, 0, 0 );
              break;
          }
          break;
      }
      break;

    default:
      return DefWindowProc( hWnd, uMsg, wParam, lParam );
  }

  return TRUE;
}


static void DoInitReaderSettings( HWND hWnd )
{
#ifndef PDAV_SHOWGUIDE
  RECT rect;
  HWND ctl;
  int xofs;
  int yofs;
  int width;
  int height;
  int i;

  GetClientRect( hWnd, &rect );

  yofs = TAB_MARGIN;
  height = 35;

  CreateStaticControl( hWnd, g_settings->hInstance, 0,
                       L"Here you can alter the general settings for the reader.",
                       TAB_MARGIN, yofs,
                       rect.right - rect.left - 2*TAB_MARGIN,
                       height, SS_LEFT | WS_VISIBLE );

  yofs += height;
  xofs = TAB_MARGIN;
  height = 20;
  width = 50;

  CreateStaticControl( hWnd, g_settings->hInstance, 0, L"Toolbar:", xofs, yofs+4, width, height-6, SS_LEFT | WS_VISIBLE );

  xofs += width + 5;
  width = 60;

  ctl = CreateComboBox( hWnd, g_settings->hInstance, ID_TOOLBAR_POS, xofs, yofs, width, height*4,
                        CBS_DROPDOWNLIST | WS_VISIBLE | WS_VSCROLL );

  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"Top" );
  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"Bottom" );
  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"None" );

  switch( g_settings->toolbar_pos )
  {
    case TOOLBAR_TOP:    i = 0; break;
    case TOOLBAR_BOTTOM: i = 1; break;
    case TOOLBAR_NONE:   i = 2; break;
  }

  SendMessage( ctl, CB_SETCURSEL, i, 0 );

  yofs += height+5;
  xofs = TAB_MARGIN;
  width = 90;

  CreateStaticControl( hWnd, g_settings->hInstance, 0, L"Screen Division:", xofs, yofs+4, width, height-6, SS_LEFT | WS_VISIBLE );

  xofs += width + 5;
  width = 70;

  ctl = CreateComboBox( hWnd, g_settings->hInstance, ID_SCREEN_DIVISIONS, xofs, yofs, width, height*4,
                        CBS_DROPDOWNLIST | WS_VISIBLE | WS_VSCROLL );

  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"None" );
  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"Quadrants" );
  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"Vertical" );
  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"Horizontal" );

  g_screen_division = g_settings->screen_division;
  SendMessage( ctl, CB_SETCURSEL, g_settings->screen_division, 0 );

  yofs += height+5;
  xofs = TAB_MARGIN + 110;
  width = 80;
  height = 15;

  g_screen_division_top = yofs + 5;

  CreateStaticControl( hWnd, g_settings->hInstance, 0, L"Tap Action:", xofs, yofs, width, height, SS_LEFT | WS_VISIBLE );

  yofs += height+2;
  height = 20;
  width = 110;

  ctl = CreateComboBox( hWnd, g_settings->hInstance, ID_TAP_ACTION, xofs, yofs, width, height*6,
                        CBS_DROPDOWNLIST | WS_VISIBLE | WS_VSCROLL );

  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"None" );
  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"Home" );
  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"Top" );
  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"Up full page" );
  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"Up 1/2 page" );
  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"Up 1/4 page" );
  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"Up one line" );
  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"Down one line" );
  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"Down 1/4 page" );
  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"Down 1/2 page" );
  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"Down full page" );
  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"Bottom" );

  EnableWindow( ctl, FALSE );

  CreateRadioButton( hWnd, g_settings->hInstance, ID_AREA_0, L"", 0, 0, 20, 20, WS_GROUP );
  CreateRadioButton( hWnd, g_settings->hInstance, ID_AREA_1, L"", 0, 0, 20, 20, 0 );
  CreateRadioButton( hWnd, g_settings->hInstance, ID_AREA_2, L"", 0, 0, 20, 20, 0 );
  CreateRadioButton( hWnd, g_settings->hInstance, ID_AREA_3, L"", 0, 0, 20, 20, 0 );

  DrawScreenDivision( hWnd, NULL );

  g_tap_actions[0] = g_settings->tap_actions[0];
  g_tap_actions[1] = g_settings->tap_actions[1];
  g_tap_actions[2] = g_settings->tap_actions[2];
  g_tap_actions[3] = g_settings->tap_actions[3];

  yofs += height;
  ctl = CreateCheckBox( hWnd, g_settings->hInstance, ID_FOLLOW_FIRST_LINK, L"Auto Link Up",
                        xofs, yofs, width, height, WS_VISIBLE );

  if( g_settings->auto_follow_first_link )
  {
    SendMessage( ctl, BM_SETCHECK, BST_CHECKED, 0 );
  }

  yofs += height;
  ctl = CreateCheckBox( hWnd, g_settings->hInstance, ID_FOLLOW_LAST_LINK, L"Auto Link Down",
                        xofs, yofs, width, height, WS_VISIBLE );

  if( g_settings->auto_follow_last_link )
  {
    SendMessage( ctl, BM_SETCHECK, BST_CHECKED, 0 );
  }

  yofs += height;
  height = 16;
  ctl = CreateStaticControl( hWnd, g_settings->hInstance, 0, L"Progress Meter:", xofs, yofs,
                             width, height, SS_LEFT | WS_VISIBLE );

  yofs += height;
  height = 20;

  ctl = CreateComboBox( hWnd, g_settings->hInstance, ID_PROGRESS_METER, xofs, yofs, width, height*3,
                        CBS_DROPDOWNLIST | WS_VISIBLE | WS_VSCROLL );

  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"None" );
  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"Left" );
  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"Right" );
  SendMessage( ctl, CB_SETCURSEL, g_settings->progress_meter, 0 );

  yofs += height + 5;
  height = 16;
  xofs = TAB_MARGIN;
  width = 95;

  ctl = CreateStaticControl( hWnd, g_settings->hInstance, 0, L"Toolbar-Toggle:", xofs, yofs,
                             width, height, SS_LEFT | WS_VISIBLE );

  xofs += width;
  yofs -= 2;
  height = 20;
  width = 110;

  ctl = CreateComboBox( hWnd, g_settings->hInstance, ID_TOOLBAR_TOGGLE_HOTSPOT, xofs, yofs, width, height*5,
                        CBS_DROPDOWNLIST | WS_VISIBLE | WS_VSCROLL );

  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"None" );
  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"Top-Left" );
  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"Top-Right" );
  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"Bottom-Left" );
  SendMessage( ctl, CB_ADDSTRING, 0, (LPARAM)L"Bottom-Right" );

  SendMessage( ctl, CB_SETCURSEL, g_settings->toolbar_hotspot, 0 );
#endif
}


static HWND CreateReaderSettingsPage( HWND hParent, LPRECT rect )
{
  HWND hWnd;
  TCHAR szWindowClass[ 64 ];

  LoadString( g_settings->hInstance, IDS_READER_SETTINGS_CLASS, szWindowClass, 64 );
  hWnd = CreateWindow( szWindowClass, L"", WS_CHILD,
                       rect->left, rect->top, rect->right - rect->left, rect->bottom - rect->top,
                       hParent, (HMENU)NULL, g_settings->hInstance, (LPVOID)WID_READER_READER );

  return hWnd;
}



static void DoSaveReaderSettings( HWND hWnd )
{
#ifndef PDAV_SHOWGUIDE
  HWND ctl;
  int  i;

  ctl = GetDlgItem( hWnd, ID_TOOLBAR_POS );
  i = SendMessage( ctl, CB_GETCURSEL, 0, 0 );

  switch( i )
  {
    case 0: i = TOOLBAR_TOP; break;
    case 1: i = TOOLBAR_BOTTOM; break;
    case 2: i = TOOLBAR_NONE; break;
  }

  if( i != g_settings->toolbar_pos )
  {
    g_settings_changed |= RS_DISPLAY_PARMS_CHANGED;
    g_settings->toolbar_pos = i;
  }

  ctl = GetDlgItem( hWnd, ID_SCREEN_DIVISIONS );
  i = SendMessage( ctl, CB_GETCURSEL, 0, 0 );

  if( i != g_settings->screen_division )
  {
    g_settings_changed |= RS_INCONSEQUENTIAL_CHANGE;
    g_settings->screen_division = i;
  }

  if( i != SCREEN_DIV_NONE )
  {
    for( i = 0; i < 4; i++ )
    {
      if( g_tap_actions[i] != g_settings->tap_actions[i] )
      {
        g_settings_changed |= RS_INCONSEQUENTIAL_CHANGE;
        g_settings->tap_actions[i] = g_tap_actions[i];
      }
    }
  }

  ctl = GetDlgItem( hWnd, ID_FOLLOW_FIRST_LINK );
  i = ( SendMessage( ctl, BM_GETCHECK, 0, 0 ) == BST_CHECKED );
  if( g_settings->auto_follow_first_link != i )
  {
    g_settings_changed |= RS_INCONSEQUENTIAL_CHANGE;
    g_settings->auto_follow_first_link = i;
  }

  ctl = GetDlgItem( hWnd, ID_FOLLOW_LAST_LINK );
  i = ( SendMessage( ctl, BM_GETCHECK, 0, 0 ) == BST_CHECKED );
  if( g_settings->auto_follow_last_link != i )
  {
    g_settings_changed |= RS_INCONSEQUENTIAL_CHANGE;
    g_settings->auto_follow_last_link = i;
  }

  ctl = GetDlgItem( hWnd, ID_PROGRESS_METER );
  i = SendMessage( ctl, CB_GETCURSEL, 0, 0 );
  if( g_settings->progress_meter != i )
  {
    g_settings_changed |= RS_CHANGE_NEEDS_REFRESH;
    g_settings->progress_meter = i;
  }

  ctl = GetDlgItem( hWnd, ID_TOOLBAR_TOGGLE_HOTSPOT );
  i = SendMessage( ctl, CB_GETCURSEL, 0, 0 );
  if( g_settings->toolbar_hotspot != i )
  {
    g_settings_changed |= RS_CHANGE_NEEDS_REFRESH;
    g_settings->toolbar_hotspot = i;
  }
#endif
}


static void DrawScreenDivision( HWND hWnd, HDC dc )
{
  BOOL need_to_release_dc = FALSE;
  RECT rect;
  HWND ctl;
  POINT line[2];

  if( dc == NULL )
  {
    need_to_release_dc = TRUE;
    dc = GetDC( hWnd );
  }

  ShowWindow( GetDlgItem( hWnd, ID_AREA_0 ), SW_HIDE );
  ShowWindow( GetDlgItem( hWnd, ID_AREA_1 ), SW_HIDE );
  ShowWindow( GetDlgItem( hWnd, ID_AREA_2 ), SW_HIDE );
  ShowWindow( GetDlgItem( hWnd, ID_AREA_3 ), SW_HIDE );

  rect.left = TAB_MARGIN;
  rect.top = g_screen_division_top;
  rect.right = rect.left + 100;
  rect.bottom = rect.top + 100;

  rect.right++;
  rect.bottom++;

  FillRect( dc, &rect, (HBRUSH)GetStockObject( WHITE_BRUSH ) );

  rect.right--;
  rect.bottom--;

  if( g_screen_division != SCREEN_DIV_NONE )
  {
    HPEN old_pen;
    HPEN new_pen;

    new_pen = CreatePen( PS_SOLID, 2, RGB( 128, 0, 128 ) );
    old_pen = SelectObject( dc, new_pen );

    Rectangle( dc, rect.left, rect.top, rect.right, rect.bottom );

    switch( g_screen_division )
    {
      case SCREEN_DIV_QUADRANTS:
        line[0].x = ( rect.right + rect.left ) / 2;
        line[0].y = rect.top;
        line[1].x = line[0].x;
        line[1].y = rect.bottom;
        Polyline( dc, line, 2 );
        line[0].x = rect.left;
        line[0].y = ( rect.bottom + rect.top ) / 2;
        line[1].x = rect.right;
        line[1].y = line[0].y;
        Polyline( dc, line, 2 );

        MoveWindow( GetDlgItem( hWnd, ID_AREA_0 ), rect.left + 15, rect.top + 15, 20, 20, TRUE );
        MoveWindow( GetDlgItem( hWnd, ID_AREA_1 ), rect.left + 65, rect.top + 15, 20, 20, TRUE );
        MoveWindow( GetDlgItem( hWnd, ID_AREA_2 ), rect.left + 15, rect.top + 65, 20, 20, TRUE );
        MoveWindow( GetDlgItem( hWnd, ID_AREA_3 ), rect.left + 65, rect.top + 65, 20, 20, TRUE );
        break;

      case SCREEN_DIV_VERTICAL:
        line[0].x = rect.left; line[1].x = rect.right;
        line[0].y = rect.top + 25; line[1].y = line[0].y;
        Polyline( dc, line, 2 );
        line[0].y = rect.top + 50; line[1].y = line[0].y;
        Polyline( dc, line, 2 );
        line[0].y = rect.top + 75; line[1].y = line[0].y;
        Polyline( dc, line, 2 );

        MoveWindow( GetDlgItem( hWnd, ID_AREA_0 ), rect.left + 40, rect.top +  3, 20, 20, TRUE );
        MoveWindow( GetDlgItem( hWnd, ID_AREA_1 ), rect.left + 40, rect.top + 28, 20, 20, TRUE );
        MoveWindow( GetDlgItem( hWnd, ID_AREA_2 ), rect.left + 40, rect.top + 53, 20, 20, TRUE );
        MoveWindow( GetDlgItem( hWnd, ID_AREA_3 ), rect.left + 40, rect.top + 78, 20, 20, TRUE );
        break;

      case SCREEN_DIV_HORIZONTAL:
        line[0].y = rect.top; line[1].y = rect.bottom;
        line[0].x = rect.left + 25; line[1].x = line[0].x;
        Polyline( dc, line, 2 );
        line[0].x = rect.left + 50; line[1].x = line[0].x;
        Polyline( dc, line, 2 );
        line[0].x = rect.left + 75; line[1].x = line[0].x;
        Polyline( dc, line, 2 );

        MoveWindow( GetDlgItem( hWnd, ID_AREA_0 ), rect.left +  4, rect.top + 40, 20, 20, TRUE );
        MoveWindow( GetDlgItem( hWnd, ID_AREA_1 ), rect.left + 29, rect.top + 40, 20, 20, TRUE );
        MoveWindow( GetDlgItem( hWnd, ID_AREA_2 ), rect.left + 54, rect.top + 40, 20, 20, TRUE );
        MoveWindow( GetDlgItem( hWnd, ID_AREA_3 ), rect.left + 79, rect.top + 40, 20, 20, TRUE );
        break;
    }

    SelectObject( dc, old_pen );
    DeleteObject( new_pen );

    if( SendMessage( GetDlgItem( hWnd, ID_AREA_0 ), BM_GETCHECK, 0, 0 ) == BST_CHECKED )
    {
      UpdateTapActionList( hWnd, 0 );
    }
    else if( SendMessage( GetDlgItem( hWnd, ID_AREA_1 ), BM_GETCHECK, 0, 0 ) == BST_CHECKED )
    {
      UpdateTapActionList( hWnd, 1 );
    }
    else if( SendMessage( GetDlgItem( hWnd, ID_AREA_2 ), BM_GETCHECK, 0, 0 ) == BST_CHECKED )
    {
      UpdateTapActionList( hWnd, 2 );
    }
    else if( SendMessage( GetDlgItem( hWnd, ID_AREA_3 ), BM_GETCHECK, 0, 0 ) == BST_CHECKED )
    {
      UpdateTapActionList( hWnd, 3 );
    }

    ShowWindow( GetDlgItem( hWnd, ID_AREA_0 ), SW_SHOW );
    ShowWindow( GetDlgItem( hWnd, ID_AREA_1 ), SW_SHOW );
    ShowWindow( GetDlgItem( hWnd, ID_AREA_2 ), SW_SHOW );
    ShowWindow( GetDlgItem( hWnd, ID_AREA_3 ), SW_SHOW );
  }
  else
  {
    ctl = GetDlgItem( hWnd, ID_TAP_ACTION );
    SendMessage( ctl, CB_SETCURSEL, -1, 0 );
    EnableWindow( ctl, FALSE );
  }

  if( need_to_release_dc )
  {
    ReleaseDC( hWnd, dc );
  }
}


static void UpdateTapActionList( HWND hWnd, int area )
{
  HWND ctl;

  g_active_area = area;
  ctl = GetDlgItem( hWnd, ID_TAP_ACTION );

  EnableWindow( ctl, TRUE );
  SendMessage( ctl, CB_SETCURSEL, g_tap_actions[ area ], 0 );
}


static HWND CreateMiscSettingsPage( HWND hParent, LPRECT rect )
{
  HWND hWnd;
  TCHAR szWindowClass[ 64 ];

  LoadString( g_settings->hInstance, IDS_READER_SETTINGS_CLASS, szWindowClass, 64 );
  hWnd = CreateWindow( szWindowClass, L"", WS_CHILD,
                       rect->left, rect->top, rect->right - rect->left, rect->bottom - rect->top,
                       hParent, (HMENU)NULL, g_settings->hInstance, (LPVOID)WID_READER_MISC );

  return hWnd;
}

static BOOL MiscSettingsProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
  switch( uMsg )
  {
  case WM_CREATE:
    DoInitMiscSettings( hWnd );
    break;

  case WM_SAVE_DATA:
    DoSaveMiscSettings( hWnd );
    break;

  default:
    return DefWindowProc( hWnd, uMsg, wParam, lParam );
  }
  return TRUE;
}

static void DoInitMiscSettings( HWND hWnd )
{
  RECT rect;
  int yofs;
  int xofs;
  int width;
  int height;
  HWND ctl;

  GetClientRect( hWnd, &rect );

  yofs = TAB_MARGIN;
  xofs = TAB_MARGIN;
  height = 35;

  CreateStaticControl( hWnd, g_settings->hInstance, 0,
                       L"Here are options that don't fit in their own category yet.",
                       xofs, yofs, rect.right - rect.left - 2*TAB_MARGIN, height, SS_LEFT | WS_VISIBLE );

  yofs += height;
  height = 20;
  width = rect.right - rect.left - 2*TAB_MARGIN;

  ctl = CreateCheckBox( hWnd, g_settings->hInstance, ID_FIRST_PARAGRAPH_IS_TITLE, L"Use first paragraph as title",
                        xofs, yofs, width, height, WS_VISIBLE );

  if( g_settings->first_paragraph_is_title )
  {
    SendMessage( ctl, BM_SETCHECK, BST_CHECKED, 0 );
  }

  yofs += height;
  ctl = CreateCheckBox( hWnd, g_settings->hInstance, ID_ENABLE_SOFT_HYPHENS, L"Enable soft hyphens",
                        xofs, yofs, width, height, WS_VISIBLE );
  SendMessage( ctl, BM_SETCHECK, g_settings->options.enable_soft_hyphens ? BST_CHECKED : BST_UNCHECKED, 0 );
}

static void DoSaveMiscSettings( HWND hWnd )
{
  HWND ctl;
  int i;

  ctl = GetDlgItem( hWnd, ID_FIRST_PARAGRAPH_IS_TITLE );
  i = ( SendMessage( ctl, BM_GETCHECK, 0, 0 ) == BST_CHECKED );
  if( g_settings->first_paragraph_is_title != i )
  {
    g_settings_changed |= RS_INCONSEQUENTIAL_CHANGE;
    g_settings->first_paragraph_is_title = i;
  }

  ctl = GetDlgItem(hWnd, ID_ENABLE_SOFT_HYPHENS );
  i = ( SendMessage( ctl, BM_GETCHECK, 0, 0 ) == BST_CHECKED );
  if( g_settings->options.enable_soft_hyphens != i )
  {
    g_settings_changed |= RS_DISPLAY_PARMS_CHANGED;
    g_settings->options.enable_soft_hyphens = i;
  }
}