/*>=-----------------------------------------------------------------------=<*
 * Vade Mecum: reader settings module
 * Copyright 2003 by Jamis Buck
 * ------------------------------------------------------------------------
 * This file is part of Vade Mecum.
 *
 * Vade Mecum is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Vade Mecum is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Vade Mecum; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 * ------------------------------------------------------------------------
 * This module defines a tabbed window that prompts the user for various
 * settings and options pertaining to the library, reader, and pagination
 * routines.
 *
 * Author: Jamis Buck (jgb3@email.byu.edu)
 *>=-----------------------------------------------------------------------=<*/

#ifndef __READER_SETTINGS_H__
#define __READER_SETTINGS_H__

#include <windows.h>
#include "vade_mecum.h"


/**
 * Constants defining what KIND of changes occurred while the user was in using
 * the settings window.  These may be OR'd together to indicate multiple types
 * of changes.
 */

#define RS_METADATA_CHANGED       ( 0x00000001 )   /* the library metadata options changed */
#define RS_BOOKPATH_CHANGED       ( 0x00000002 )   /* the path(s) to the ebooks has changed */
#define RS_DISPLAY_PARMS_CHANGED  ( 0x00000004 )   /* display parameters have changed, requiring a repagination */
#define RS_INCONSEQUENTIAL_CHANGE ( 0x00000008 )   /* an inconsequential change ocurred, affecting nothing visible */
#define RS_CHANGE_NEEDS_REFRESH   ( 0x00000010 )   /* the change requires that the screen be refreshed */


/**
 * Opens and executes the reader settings window.  If the user modifies nothing or cancels the dialog, this
 * function returns 0.  Otherwise, it returns a combination of the RS_xxx constants (above), indicating
 * what kind of changes were made to the library settings.
 */
UINT32 OpenReaderSettings( HWND hWnd, Library *settings );

/**
 * The callback function for the reader settings window, used to register the window class.
 */
BOOL CALLBACK ReaderSettingsDialogProc( HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam );


#endif /* __READER_SETTINGS_H__ */
