/*>=-----------------------------------------------------------------------=<*
 * Vade Mecum: reader toolbar module
 * Copyright 2003 by Jamis Buck
 * Portions copyright 2004 by Henrik Kjoelhede
 * ------------------------------------------------------------------------
 * This file is part of Vade Mecum.
 *
 * Vade Mecum is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Vade Mecum is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Vade Mecum; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *>=-----------------------------------------------------------------------=<*/

#include <windows.h>
#include <wingdi.h>
#include <commctrl.h>
#include <aygshell.h>
#include "reader-toolbar.h"
#include "resrc1.h"
#include "reader-ui.h"  /* ID command codes */


#define BITMAP_WIDTH    ( 16 )
#define BITMAP_HEIGHT   ( 16 )

#define BUTTON_WIDTH    BITMAP_WIDTH
#define BUTTON_HEIGHT   BITMAP_HEIGHT


#define ID_PERCENTAGE_TEXT      ( 50001 )


static Library *static_library = NULL;
static int      static_cached_height = -1;
static int      static_percentage = 0;
static LOGFONT  static_font_def;


HWND CreateReaderToolbar( HWND hWnd, Library* library, Boolean showZoomInOut )
{
  HWND toolbar;
  RECT rect;
  int id;
  SHMENUBARINFO mbi;



  static_library = library;

  GetClientRect( hWnd, &rect );
  if (showZoomInOut)
  {
    id = IDR_MENUBAR_MAP;
  }
  else
  {
    id = IDR_MENUBAR_DOC;
  }

  memset(&mbi, 0, sizeof(SHMENUBARINFO));
  mbi.cbSize     = sizeof(SHMENUBARINFO);
  mbi.hwndParent = hWnd;
  mbi.nToolBarId = id;
  mbi.hInstRes   = library->hInstance;

  if (!SHCreateMenuBar(&mbi)) 
  {
      toolbar = NULL;
  }
  else
  {
      toolbar = mbi.hwndMB;
      CommandBar_AddBitmap(toolbar,library->hInstance, IDB_TOOLBAR_BUTTONS, 9, BITMAP_WIDTH, BITMAP_HEIGHT);
  }

  return toolbar;
}


UINT DrawToolbar( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
#ifndef PDAV_SHOWGUIDE
  LPNMCUSTOMDRAW drawdata = (LPNMCUSTOMDRAW)lParam;
  HDC dc;
  RECT rect;


  switch( drawdata->dwDrawStage )
  {
    case CDDS_PREPAINT:
      /* request per-item redraw callback */
      dc = GetDC( hWnd );
      GetClientRect( hWnd, &rect );
      DrawEdge( dc, &rect, 
                ( static_library->toolbar_pos == TOOLBAR_TOP ? EDGE_RAISED : EDGE_SUNKEN ),
                ( static_library->toolbar_pos == TOOLBAR_TOP ? BF_BOTTOM : BF_TOP ) );
      ReleaseDC( hWnd, dc );
      return CDRF_NOTIFYITEMDRAW;

    case CDDS_ITEMPREPAINT:
      return CDRF_DODEFAULT;
  }
#endif
  return 0;
}


void SetViewedPercentage( HWND hWnd, int pct )
{
  if( pct < 0 ) pct = 0;
  if( pct > 100 ) pct = 100;

  if( static_percentage == pct ) return;

  static_percentage = pct;
  InvalidateRect( hWnd, NULL, TRUE );
}


UINT GetReaderToolbarHeight( HWND hToolbar )
{
  RECT rect;

  if( hToolbar == NULL || static_library == NULL ) return 0;
#ifndef PDAV_SHOWGUIDE
  if( static_library->toolbar_pos == TOOLBAR_NONE ) return 0;
#endif
  if( static_cached_height >= 0 ) return static_cached_height;

  GetClientRect( hToolbar, &rect );
  static_cached_height = rect.bottom - rect.top;
  return static_cached_height;
}


void MoveToolbarPosition( HWND hToolbar, int pos )
{
#ifndef PDAV_SHOWGUIDE
  int style;

  style = SendMessage( hToolbar, TB_GETSTYLE, 0, 0 );
  style &= ~( CCS_TOP | CCS_BOTTOM );

  switch( pos )
  {
    case TOOLBAR_TOP:
      style |= CCS_TOP;
      break;

    case TOOLBAR_BOTTOM:
      style |= CCS_BOTTOM;
      break;
  }

  SendMessage( hToolbar, TB_SETSTYLE, 0, style );
#endif
}


void EnableToolbarButton( HWND hToolbar, int button, BOOL enable )
{
  SendMessage( hToolbar, TB_ENABLEBUTTON, button, MAKELONG(enable,0) );
}

void CheckToolbarButton( HWND hToolbar, int button, BOOL check )
{
  SendMessage( hToolbar, TB_CHECKBUTTON, button, MAKELONG(check,0) );
}

BOOL IsToolbarButtonChecked( HWND hToolbar, int button )
{
  TBBUTTON tb;
  SendMessage( hToolbar, TB_GETBUTTON, button, (LPARAM)&tb );
  return ( tb.fsStyle | TBSTATE_CHECKED );
}