/*>=-----------------------------------------------------------------------=<*
 * Vade Mecum: reader toolbar module
 * Copyright 2003 by Jamis Buck
 * Portions copyright 2004 by Henrik Kjoelhede
 * ------------------------------------------------------------------------
 * This file is part of Vade Mecum.
 *
 * Vade Mecum is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Vade Mecum is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Vade Mecum; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 * ------------------------------------------------------------------------
 * This module defines a toolbar to be used in the reader, which can be
 * positioned either at the top of the screen or the bottom.  The provided
 * routines are the exported API that may be used to affect the toolbar.
 *
 * Author: Jamis Buck (jgb3@email.byu.edu)
 *>=-----------------------------------------------------------------------=<*/

#ifndef __READER_TOOLBAR_H__
#define __READER_TOOLBAR_H__

#include "library.h"

/**
 * Creates a new toolbar object and returns a handle to it.
 */
HWND CreateReaderToolbar( HWND hWnd, Library* library, Boolean showZoomInOut );

/**
 * Draws the toolbar on the given window, either at the top or the bottom, as
 * requested.
 */
UINT DrawToolbar( HWND hWnd, WPARAM wParam, LPARAM lParam );

/**
 * Sets the percentage of the current document that has been viewed, so that the
 * toolbar's percentage display can be updated.
 */
void SetViewedPercentage( HWND hWnd, int pct );

/**
 * Returns the height of the toolbar, in pixels.
 */
UINT GetReaderToolbarHeight( HWND hToolbar );

/**
 * Moves the toolbar to the top, or the bottom, of the window.  'pos' must be
 * either TOOLBAR_TOP or TOOLBAR_BOTTOM (as defined in library.h).
 */
void MoveToolbarPosition( HWND hToolbar, int pos );

/**
 * Causes the given button (identified by command value) to be enabled or disabled.
 */
void EnableToolbarButton( HWND hToolbar, int button, BOOL enable );

void CheckToolbarButton( HWND hToolbar, int button, BOOL check );

BOOL IsToolbarButtonChecked( HWND hToolbar, int button );

#endif /* __READER_TOOLBAR_H__ */
