/*>=-----------------------------------------------------------------------=<*
 * Vade Mecum: SearchDialog module
 * Copyright 2003 by Jamis Buck
 * Portions copyright 2004 by Henrik Kjoelhede
 * ------------------------------------------------------------------------
 * This file is part of Vade Mecum.
 *
 * Vade Mecum is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Vade Mecum is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Vade Mecum; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 * ------------------------------------------------------------------------
 * This creates a window that allows users to search for a given pattern
 * of text in a plucker file.
 *
 * Author: Jamis Buck (jgb3@email.byu.edu)
 *>=-----------------------------------------------------------------------=<*/

#ifndef __SEARCH_DLG_H__
#define __SEARCH_DLG_H__

#include <windows.h>
#include "vade_mecum.h"
#include "plkfmt.h"

typedef struct
{
  int record;
  int text_offset;
  int paragraph;
} SearchMatch;

typedef struct
{
  int record;
  int text_offset;
  int paragraph;
  int flags;
  BOOL just_this_record;
  char pattern[ MAX_SEARCH_PATTERN ];
} SearchReturn;

/**
 * Registers the SearchDialog window class.  This should be called before SearchDialog() is ever called.
 */
ATOM RegisterSearchDialog( HINSTANCE hInst );

/**
 * Opens a search dialog which will allow the user to search either the currently open record (if >= 0),
 * or all records in the given PluckerDB handle (if non-NULL).
 *
 * Returns FALSE if the user presses cancel, TRUE if the user selects a match from the list and chooses
 * view it.  In this case, the 'selected' parameter will be populated with the information appropriate
 * for that match.
 */
BOOL SearchDialog( HWND parent, Library *library, PluckerDB db, int record, SearchReturn *selected );

#endif /* __SEARCH_DLG_H__ */
