/*>=-----------------------------------------------------------------------=<*
 * Vade Mecum: main header file
 * Copyright 2003 by Jamis Buck
 * Portions copyright 2004 by Henrik Kjoelhede
 * ------------------------------------------------------------------------
 * This file is part of Vade Mecum.
 *
 * Vade Mecum is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Vade Mecum is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Vade Mecum; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 * ------------------------------------------------------------------------
 * This header file defines constants and structures used by many of the
 * other modules in Vade Mecum.
 *
 * Author: Jamis Buck (jgb3@email.byu.edu)
 *>=-----------------------------------------------------------------------=<*/

#ifndef __VADE_MECUM_H__
#define __VADE_MECUM_H__

#include <windows.h>     /* HINSTANCE, HWND, TCHAR, etc. */
#include "pagination.h"  /* FormattingDefaults */

#include "plkfmt.h"      /* Plucker database interface */

#include "history.h"

/* this is a hack to support cleartype, since the headers that come eVC++ 3 don't
 * seem to include the necessary values... :( */
#define USE_CLEARTYPE_QUALITY  ( 5 )

/* these are more hacks to support missing constants... */
#define VADE_SPI_GETFONTSMOOTHING       ( 74 )
#define VADE_SPI_SETFONTSMOOTHING       ( 75 )
#define VADE_SPI_GETFONTSMOOTHINGTYPE   ( 0x200A )
#define VADE_SPI_SETFONTSMOOTHINGTYPE   ( 0x200B )

#define VADE_FE_FONTSMOOTHINGSTANDARD   ( 0x0001 )
#define VADE_FE_FONTSMOOTHINGCLEARTYPE  ( 0x0002 )
#define VADE_FE_FONTSMOOTHINGDOCKING    ( 0x8000 )

#define VADE_SPI_GETFONTSMOOTHINGCONTRAST    ( 0x200C )
#define VADE_SPI_SETFONTSMOOTHINGCONTRAST    ( 0x200D )



#define MAX_EBOOK_PATHS_LEN    ( 1024 )    /* maximum length of the ebook-paths member */
#define MAX_CATEGORY_NAME_LEN  (   32 )    /* maximum length of a category name */
#define MAX_CATEGORIES         (   16 )    /* the maximum number of categories */
#define MAX_METADATA_PATH_LEN  (  256 )    /* maximum length of the meta-data paths member */
#define MAX_DOCUMENT_ID_LEN    (    9 )    /* maximum document id length */
#define MAX_DOCUMENT_TITLE_LEN (   33 )    /* maximum length of a document title */
#define MAX_SEARCH_PATTERN     (  256 )    /* maximum length of a search pattern */
#define NUMBER_RECENT_SEARCHES (    5 )

/* version information */
#define VADE_MECUM_VERSION_MAJOR    (  0 )
#define VADE_MECUM_VERSION_MINOR    (  6 )
#define VADE_MECUM_VERSION          ( ( VADE_MECUM_VERSION_MAJOR << 8 ) | VADE_MECUM_VERSION_MINOR )

/* screen division constants */
#define SCREEN_DIV_NONE             (  0 )
#define SCREEN_DIV_QUADRANTS        (  1 )
#define SCREEN_DIV_VERTICAL         (  2 )
#define SCREEN_DIV_HORIZONTAL       (  3 )

/**
 * tap actions (the order of these constants should not be changed, since the settings
 * window depends on them being in this order, with the values as given.  New constants
 * should be added at the end.)
 */
#define TAP_ACTION_NONE               (  0 )
#define TAP_ACTION_HOME               (  1 )
#define TAP_ACTION_TOP                (  2 )
#define TAP_ACTION_UP_FULLPAGE        (  3 )
#define TAP_ACTION_UP_HALFPAGE        (  4 )
#define TAP_ACTION_UP_QUARTERPAGE     (  5 )
#define TAP_ACTION_UP_ONELINE         (  6 )
#define TAP_ACTION_DOWN_ONELINE       (  7 )
#define TAP_ACTION_DOWN_QUARTERPAGE   (  8 )
#define TAP_ACTION_DOWN_HALFPAGE      (  9 )
#define TAP_ACTION_DOWN_FULLPAGE      ( 10 )
#define TAP_ACTION_BOTTOM             ( 11 )


#ifndef PDAV_SHOWGUIDE
/* toolbar position info */
typedef enum
{
  TOOLBAR_TOP,
  TOOLBAR_BOTTOM,
  TOOLBAR_NONE
} ToolbarPosition;

/* toolbar toggle hotspot position */
typedef enum
{
  HOTSPOT_NONE = 0,
  HOTSPOT_TOPLEFT,
  HOTSPOT_TOPRIGHT,
  HOTSPOT_BOTTOMLEFT,
  HOTSPOT_BOTTOMRIGHT
} ToolbarToggleHotspot;

#endif

/* graphic progress meter info */
typedef enum
{
  PROGRESS_METER_NONE = 0,
  PROGRESS_METER_LEFT,
  PROGRESS_METER_RIGHT
} ProgressMeter;


/* The CategoryDefinition type defines a single ebook category. */
typedef struct
{
  TCHAR name[ MAX_CATEGORY_NAME_LEN ];  /* name of the category */
  BYTE  id;                             /* the id (0-15) of the category */ 
} CategoryDefinition;


/* The Document type defines the meta-data about a specific document. */
typedef struct
{
  TCHAR  filename[ MAX_PATH ];         /* the document's filename (full-path) */
  TCHAR  title[ MAX_DOCUMENT_TITLE_LEN ]; /* the document's title */
  UINT16 categories;                   /* a bitfield containing the categories that this file belongs to */
  UINT32 size;                         /* the size of the file, in bytes */
  BOOL   verified;                     /* whether this document has been verified or not */
  FILETIME created;                    /* the date that this file was last created */
} Document;

/* Store recent searches for easy researching */
typedef struct
{
  char pattern[ MAX_SEARCH_PATTERN ];
  int index;
} RecentSearch;

/* The Library type defines the data that is global to the entire library.  It also defines
 * information about the library, such as categories and ebook-paths. */
typedef struct
{
  UINT16 size;                                     /* the structure size */
  UINT16 version;                                  /* Vade Mecum version (HIBYTE->major, LOBYTE->minor) */
  TCHAR ebook_paths[ MAX_EBOOK_PATHS_LEN ];        /* semi-colon delimited list of paths to ebook directories */
  FormattingDefaults options;                      /* default book-formatting options */
  CategoryDefinition categories[ MAX_CATEGORIES ]; /* the list of categories */
  TCHAR metadata_path[ MAX_METADATA_PATH_LEN];     /* a single path naming a directory to store library metadata info in */
  BOOL use_hscroll;                                /* whether or not the reader windows should use a horizontal scrollbar, as needed */
  BOOL use_vscroll;                                /* whether or not the reader windows should use a vertical scrollbar, as needed */
#ifndef PDAV_SHOWGUIDE
  ToolbarPosition toolbar_pos;                     /* where to position the toolbar */
#endif
  UINT8 pagination_cache_size;                     /* number of records in the pagination cache (PER DOCUMENT!!!) */
  UINT8 screen_division;                           /* the division of the screen, for determining tap behavior */
  UINT8 tap_actions[4];                            /* the action to perform for each of the 4 possible screen divisions */
  UINT8 autoscroll_delay;                          /* hundredths of a second (10-millisecond increments) */
  UINT8 autoscroll_amount;                         /* # of pixels to scroll */

  UINT16 current_categories;                       /* the bit-field identifying the categories currently being displayed */

  COLORREF fg_colors[8];                           /* the 8 customizable foreground colors for highlights */
  COLORREF bg_colors[8];                           /* the 8 customizable background colors for highlights */
  BOOL     foreground_selected;                    /* whether or not a foreground color has been selected */
  BOOL     background_selected;                    /* whether or not a background color has been selected */
  UINT32   foreground : 24;                        /* the foreground color (0xBBGGRR) */
  UINT32   background : 24;                        /* the background color (0xBBGGRR) */

  UINT8 history_size;                              /* # of history records to keep PER DOCUMENT */
  HISTORY_LIST *history;

  BOOL      full_screen;                           /* whether or not the reader should be displayed full-screen or not */
  INT8      reader_sort_column;                    /* which column to sort on -- negative means inverse sort, 0 means no sort */
  BOOL      auto_follow_first_link;                /* follow first link on page on scrolling beyond top of page */
  BOOL      auto_follow_last_link;                 /* follow the last link on the page on scrolling beyond the bottom of page */

  TCHAR     current_document[ MAX_PATH ];          /* the name of the current document, or "" if there is no current one */

  ProgressMeter progress_meter;                    /* where the graphic progress meter should be placed */
#ifndef PDAV_SHOWGUIDE
  BOOL      toolbar_hidden;                        /* whether or not the toolbar is hidden */
  ToolbarToggleHotspot toolbar_hotspot;            /* where the user may tap on the screen to toggle the toolbar */
#else
  TCHAR     home_document[ MAX_PATH ];             /* the name of the initially launched document */
#endif
  Document *document_cache;                        /* cache of document meta-data entries */
  UINT16    document_cache_size;                   /* # of elements in the cache */

  FontCache font_cache;                            /* the font-cache to be used by the reader */

  HINSTANCE hInstance;                             /* the module instance containing all of this */
  HWND      hWnd;                                  /* the window that owns all of the reader windows */

  RecentSearch recent_searches[ NUMBER_RECENT_SEARCHES ];

  BOOL      first_paragraph_is_title;

} Library;

#endif /* __VADE_MECUM_H__ */
